package com.singularity.controller;

import cn.dev33.satoken.exception.SaTokenException;
import cn.dev33.satoken.util.SaResult;
import com.singularity.pojo.param.kc.ExportKeyIndexParam;
import com.singularity.pojo.param.kc.SaveExcelParam;
import com.singularity.service.CarKeyIndexService;
import com.singularity.service.impl.CarKeyIndexServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

@Slf4j
@RestController
@Api(tags = "carKeyIndex相关接口")
@RequestMapping(value = "/api/carKeyIndex")
public class CarKeyIndexController {


    @Resource
    private CarKeyIndexService carKeyIndexService;

    @ApiOperation("存储excel数据")
    @PostMapping(value = "/saveExcel")
    public SaResult saveExcel(@RequestBody SaveExcelParam saveExcelParam) throws FileNotFoundException {
        carKeyIndexService.saveExcel(saveExcelParam);
        return SaResult.ok();
    }

    @ApiOperation("导出关键指标")
    @PostMapping(value = "/export")
    public void exportKeyIndex(HttpServletResponse response, @RequestBody ExportKeyIndexParam exportKeyIndexParam) throws FileNotFoundException {
        InputStream inputStream=carKeyIndexService.exportKeyIndex(exportKeyIndexParam);
        String fileName = CarKeyIndexServiceImpl.class.getResource("/").getPath() + "Write" + ".xlsx";
        if(Objects.isNull(inputStream)){
            throw new SaTokenException("下载失败");
        }
        try {
            // 字符集编码
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            //清空response中的输出流
            response.reset();
            //设置Content-Type头
            response.setContentType("application/octet-stream;charset=UTF-8");
            //下面这行代码必须加，否则前端获取不到Content-Disposition字段，即无法获取文件名
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            BufferedInputStream bis = new BufferedInputStream(inputStream);
            BufferedOutputStream bos = new BufferedOutputStream(response.getOutputStream());
            byte[] buffer = new byte[1024];
            int length;
            while ((length = bis.read(buffer)) != -1) {
                bos.write(buffer, 0, length);
            }
            bis.close();
            bos.close();
        } catch (IOException e) {
            throw new SaTokenException("下载失败");
        }
//        return new SaResult(SaResult.CODE_SUCCESS,"导出成功",inputStream);
    }





}
