package com.singularity.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.singularity.pojo.dto.condition.*;
import com.singularity.pojo.vo.condition.ConditionInfoListQueryVO;
import com.singularity.pojo.vo.condition.ConditionNameVO;
import com.singularity.service.ConditionInfoService;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@Api(tags = "工况管理相关接口")
@RestController
@RequestMapping("/api/condition")
public class ConditionInfoController {

    @Resource
    private ConditionInfoService conditionInfoService;


    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("工况列表分页查询")
    @PostMapping("/queryConditionInfoList")
    public Result<List<ConditionInfoListQueryVO>> queryConditionInfoList(@RequestBody @Validated ConditionInfoListQueryDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,conditionInfoService.queryConditionInfoList(dto));
    }

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("增加工况信息")
    @PostMapping("/addConditionInfo")
    public Result<?> addConditionInfo(@RequestBody @Validated ConditionInfoAddDTO dto) {
        conditionInfoService.ddConditionInfo(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("删除工况信息")
    @DeleteMapping("/deleteConditionInfo")
    public Result<?> deleteConditionInfo(@RequestBody @Validated ConditionInfoDeleteDTO dto) {
        conditionInfoService.deleteConditionInfo(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("修改工况信息")
    @PostMapping("/updateConditionInfo")
    public Result<?> updateConditionInfo(@RequestBody @Validated ConditionInfoUpdateDTO dto) {
        conditionInfoService.updateConditionInfo(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("分类查询工况接口")
    @PostMapping("/selectConditionByType")
    public Result<List<ConditionNameVO>> selectConditionByType(@RequestBody @Validated ConditionTypeDTO conditionTypeDTO) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                conditionInfoService.selectConditionByType(conditionTypeDTO)
        );
    }
}

