package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.damper.*;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.common.entity.PartChartData;
import com.singularity.pojo.entity.DamperData;
import com.singularity.pojo.vo.damper.CompareDamperDataFileDetailVO;
import com.singularity.pojo.vo.damper.QueryDamperCompareChartDataVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.DamperChartDataService;
import com.singularity.service.DamperDataService;
import com.singularity.pojo.vo.damper.QueryDamperDataFileDetailVO;
import com.singularity.pojo.vo.damper.QueryDamperDataFileListVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/damper")
@Api(tags = "减震器调试相关接口")
public class DamperController {
    @Resource
    private AttachmentService attachmentService;

    @Resource
    private DamperChartDataService damperChartDataService;

    @Resource
    private DamperDataService damperDataService;

    @ApiOperation("上传减震器试验数据文件")
    @PostMapping(value = "/uploadDamperFile")
    public Result<?> uploadDamperFile(@Validated UploadDamperDataFileDTO dto) {
        damperChartDataService.uploadDamperDataFile(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除减震器数据文件")
    @PostMapping(value = "/deleteDamperFile")
    public Result<?> deleteDamperFile(@RequestBody @Validated DeleteDamperAttachmentDTO dto) {
        // 删除minio附件
        attachmentService.removeAttachmentById(
                dto.getAttachIdList()
                        .stream().map(Integer::longValue)
                        .collect(Collectors.toList()));
        // 删除减振器试验数据
        LambdaQueryWrapper<DamperData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(DamperData::getAttachId,dto.getAttachIdList());
        damperDataService.remove(queryWrapper);

        // 删除减振器绘图数据
        LambdaQueryWrapper<PartChartData> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.in(PartChartData::getAttachId,dto.getAttachIdList());
        damperChartDataService.remove(queryWrapper2);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询减震器试验数据文件列表")
    @PostMapping(value = "/queryDamperFile")
    public Result<List<QueryDamperDataFileListVO>> queryDamperFile(@RequestBody @Validated QueryDamperAttachmentDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryDamperFile(dto));
    }

    @ApiOperation("查询减震器试验数据文件详情")
    @PostMapping(value = "/queryDamperDataFileDetail")
    public Result<QueryDamperDataFileDetailVO> queryDamperDataFileDetail(@RequestBody @Validated QueryDamperDataFileDetailDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,damperChartDataService.queryDamperDataFileDetail(dto));
    }

    @ApiOperation("对比减震器试验数据文件详情")
    @PostMapping(value = "/compareDamperDataFileDetail")
    public Result<CompareDamperDataFileDetailVO> compareDamperDataFileDetail(@RequestBody @Validated CompareDamperDataFileDetailDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,damperChartDataService.compareDamperDataFileDetail(dto));
    }

    @ApiOperation("查询减震器对比曲线数据")
    @PostMapping(value = "/queryDamperCompareChartData")
    public Result<List<QueryDamperCompareChartDataVO>> queryDamperCompareChartData(@RequestBody @Validated QueryDamperCompareChartDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,damperChartDataService.queryDamperCompareChartData(dto));
    }


    @ApiOperation("查询减震器试验曲线数据")
    @PostMapping(value = "/queryDamperChartDataList")
    public Result<List<ChartDataModel>> queryDamperChartDataList(@RequestBody @Validated QueryDamperChartDataListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,damperChartDataService.queryDamperChartDataList(dto));
    }

    @ApiOperation("修改减震器试验数据文件详情")
    @PostMapping(value = "/updateDamperDataFileDetail")
    public Result<?> updateDamperDataFileDetail(@RequestBody @Validated UpdateDamperDataFileDetailDTO dto) {
        damperDataService.updateDamperDataFileDetail(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }


    @ApiOperation("上传减震器试验报告文件")
    @PostMapping(value = "/uploadDamperReport")
    public Result<?> uploadDamperReport(@Validated UploadDamperReportFileDTO dto) {
        // 文件名验重
        attachmentService.checkRepeatFileName(
                dto.getMultipartFile().getOriginalFilename(),
                dto.getCarId(),
                FileBusinessConstant.DAMPER_REPORT,
                AttachmentConstant.OWNER_TYPE_CAR
        );

        // 上传文件
        attachmentService.uploadAttachment(
                dto.getMultipartFile(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                FileBusinessConstant.DAMPER_REPORT,
                dto.getOperId()
        );
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除减震器实验报告")
    @PostMapping(value = "/deleteDamperReport")
    public Result<?> deleteDamperReport(@RequestBody @Validated DeleteDamperAttachmentDTO dto) {
        damperChartDataService.deleteDamperReport(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }


    @ApiOperation("行驶性能减震器试验报告主列表分页查询")
    @PostMapping(value = "/queryDamperReportFileList")
    public Result<List<QueryAttachmentListVO>> queryDamperReportFileList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(FileBusinessConstant.DAMPER_REPORT);
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }
}
