package com.singularity.controller;


import cn.dev33.satoken.util.SaResult;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.dto.department.DepartmentAddDTO;
import com.singularity.pojo.dto.department.DepartmentDeleteDTO;
import com.singularity.pojo.dto.department.DepartmentUpdateDTO;
import com.singularity.pojo.dto.department.QueryDepartmentTreeListDTO;
import com.singularity.service.DepartmentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@Slf4j
@Api(tags = "部门管理相关接口")
@RestController
@RequestMapping("/api/department")
public class DepartmentController {
    @Resource
    private DepartmentService departmentService;


    @ApiOperation("添加部门")
    @LogOperation(message = "添加部门")
    @PostMapping(value = "/addDepartmentInfo")
    public SaResult addDepartment(@RequestBody DepartmentAddDTO departmentAddDTO){
        departmentService.addDepartment(departmentAddDTO);
        return SaResult.ok("添加成功");
    }

    @ApiOperation("树状部门主列表分页查询")
    @PostMapping(value = "/queryDepartmentTreeList")
    public SaResult queryDepartmentTreeList(@Validated @RequestBody QueryDepartmentTreeListDTO queryDepartmentTreeListDTO){
        return new SaResult(SaResult.CODE_SUCCESS,"获取成功", departmentService.queryDepartmentTreeList(queryDepartmentTreeListDTO));
    }

    @ApiOperation("更新部门")
    @LogOperation(message = "更新部门")
    @PostMapping(value = "/updateDepartmentInfo")
    public SaResult updateDepartment(@RequestBody DepartmentUpdateDTO departmentUpdateDTO){
        departmentService.updateDepartment(departmentUpdateDTO);
        return SaResult.ok("修改成功");
    }


    @ApiOperation("删除部门")
    @LogOperation(message = "删除部门")
    @PostMapping(value = "/deleteDepartmentList")
    public SaResult deleteDepartmentList(@RequestBody DepartmentDeleteDTO departmentDeleteDTO){
        departmentService.deleteDepartmentList(departmentDeleteDTO);
        return SaResult.ok("删除成功");
    }
}
