package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.experiment.DeleteInertiaDTO;
import com.singularity.pojo.dto.experiment.ExperimentFileDTO;
import com.singularity.pojo.entity.SineFitCharacteristic;
import com.singularity.pojo.entity.VehicleInformation;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.SineFitCharacteristicService;
import com.singularity.service.VehicleInformationService;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@RestController
@Api(tags = "质心及转动惯量相关接口")
@RequestMapping(value = "/api/inertia")
public class InertiaController {
    @Resource
    private AttachmentService attachmentService;

    @Resource
    private SineFitCharacteristicService sineFitCharacteristicService;

    @Resource
    private VehicleInformationService vehicleInformationService;

    @ApiOperation("质心及转动惯量附件上传")
    @LogOperation(message = "质心及转动惯量附件上传")
    @PostMapping("/updateInertiaFile")
    public Result<Integer> updateInertiaFile(@NotNull @Validated ExperimentFileDTO experimentFileDTO) {
        attachmentService.asyncUpdateInertiaFile(experimentFileDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("批量删除质心及转动惯量附件")
    @LogOperation(message = "批量删除质心及转动惯量附件")
    @PostMapping(value = "/deleteInertiaFile")
    public Result<?> deleteInertiaFile(@Validated @RequestBody DeleteInertiaDTO dto) {
        //附件删除
        attachmentService.removeAttachmentById(dto.getAttachIdList());
        //表格数据删除
        LambdaUpdateWrapper<VehicleInformation> qwv = new LambdaUpdateWrapper<>();
        qwv.in(VehicleInformation::getAttachId,dto.getAttachIdList());
        vehicleInformationService.remove(qwv);
        LambdaUpdateWrapper<SineFitCharacteristic> qws = new LambdaUpdateWrapper<>();
        qws.in(SineFitCharacteristic::getAttachId,dto.getAttachIdList());
        sineFitCharacteristicService.remove(qws);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("质心及转动惯量试数据文件列表查询")
    @PostMapping(value = "/queryInertiaFileList")
    public Result<List<QueryAttachmentListVO>> queryInertiaFileList(@RequestBody @Validated QueryAttachmentListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                attachmentService.queryAttachmentList(dto));
    }
}
