package com.singularity.controller;


import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.singularity.pojo.dto.measure.*;
import com.singularity.service.MeasureInfoService;
import com.singularity.pojo.vo.subjectiveEvaluation.measure.QueryMeasureInfoListVO;
import com.singularity.pojo.vo.subjectiveEvaluation.measure.QueryMeasureListByConditionIdVO;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@Api(tags = "测点管理相关接口")
@RestController
@RequestMapping("/api/measure")
public class MeasureInfoController {

    @Resource
    private MeasureInfoService measureInfoService;

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("测点列表分页查询")
    @PostMapping("/queryMeasureInfoList")
    public Result<List<QueryMeasureInfoListVO>> queryMeasureInfoList(@RequestBody @Validated QueryMeasureInfoListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,measureInfoService.queryMeasureInfoList(dto));
    }

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("增加测点信息")
    @PostMapping("/addMeasureInfo")
    public Result<?> addMeasureInfo(@RequestBody @Validated AddMeasureInfoDTO dto) {
        measureInfoService.addMeasureInfo(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("删除测点信息")
    @DeleteMapping("/deleteMeasureInfo")
    public Result<?> deleteMeasureInfo(@RequestBody @Validated DeleteMeasureInfoDTO dto) {
        measureInfoService.deleteMeasureInfo(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("修改测点信息")
    @PostMapping("/updateMeasureInfo")
    public Result<?> updateMeasureInfo(@RequestBody @Validated UpdateMeasureInfoDTO dto) {
        measureInfoService.updateMeasureInfo(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "wan xing")
    @ApiOperation("根据工况id查询测点列表")
    @PostMapping("/queryMeasureListByConditionId")
    public Result<List<QueryMeasureListByConditionIdVO>> queryMeasureListByConditionId(@RequestBody @Validated QueryMeasureListByConditionIdDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,measureInfoService.queryMeasureListByConditionId(dto));
    }
}

