package com.singularity.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.singularity.pojo.param.role.RoleAddParam;
import com.singularity.pojo.param.role.RoleDeleteParam;
import com.singularity.pojo.param.role.RoleListSearchParam;
import com.singularity.pojo.param.role.RoleSetParam;
import com.singularity.pojo.vo.role.RoleListVO;
import com.singularity.service.RoleService;
import com.singularity.util.page.ResponseDTOWithPage;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/role")
@Api(tags = "角色相关")
public class RoleController {

    private final RoleService roleService;

    @Autowired
    public RoleController(RoleService roleService) {
        this.roleService = roleService;
    }

    @ApiOperation("分页查询角色信息列表")
    @ApiOperationSupport(author = "winsky")
    @RequestMapping(method = RequestMethod.POST, value = "/roleList")
    public Result<ResponseDTOWithPage<RoleListVO>> selectRoleList(@Validated @RequestBody RoleListSearchParam roleListSearchDTO) {
        ResponseDTOWithPage<RoleListVO> list = roleService.selectRoleList(roleListSearchDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,list);
    }

    @ApiOperation("新增角色信息")
    @ApiOperationSupport(author = "winsky")
    @RequestMapping(method = RequestMethod.POST, value = "/addRole")
    public Result<?> addRole(@RequestBody RoleAddParam roleAddParam) {
        roleService.addRole(roleAddParam);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,null);
    }

    @ApiOperation("修改角色信息")
    @ApiOperationSupport(author = "winsky")
    @RequestMapping(method = RequestMethod.POST, value = "/setRole")
    public Result<?> setRole(@RequestBody RoleSetParam roleSetParam) {
        roleService.setRole(roleSetParam);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,null);
    }

    @ApiOperation("删除角色信息")
    @ApiOperationSupport(author = "winsky")
    @RequestMapping(method = RequestMethod.POST, value = "/deleteRoles")
    public Result<?> deleteRoles(@RequestBody RoleDeleteParam roleSetParam) {
        roleService.deleteRoles(roleSetParam);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,null);
    }
}
