package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.exception.SaTokenException;
import cn.dev33.satoken.util.SaResult;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.dao.NVHChartDataMapper;
import com.singularity.pojo.dto.attachment.DeleteAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.DeleteEvaluationDataDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.QueryEvaluatorNameListDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.QueryNVHSubjectiveChartDataDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.SubjectiveIdDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.CarColumn;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.pojo.entity.NVHChartData;
import com.singularity.pojo.param.file.PreviewParam;
import com.singularity.pojo.param.subjectiveEvaluation.*;
import com.singularity.service.AttachmentService;
import com.singularity.service.CarColumnService;
import com.singularity.service.SubjectiveEvaluationService;
import com.singularity.pojo.vo.subjectiveEvaluation.GetSubjectiveEvaluationListVO;
import com.singularity.pojo.vo.subjectiveEvaluation.QueryEvaluatorNameListVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.util.FileUtil;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/subjectiveEvaluation")
@Api(tags = "主观评价相关接口")
public class SubjectiveEvaluationController {

    @Resource
    private SubjectiveEvaluationService subjectiveEvaluationService;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private CarColumnService carColumnService;

    @Resource
    private NVHChartDataMapper nvhChartDataMapper;

    @ApiOperation("上传主观评价评价结果数据文件")
    @LogOperation(message = "上传主观评价评价结果数据文件")
    @PostMapping(value = "/uploadFile")
    public SaResult UploadFile(@Validated SubjectiveEvaluationUploadFileParam subjectiveEvaluationUploadFileParam) {
        subjectiveEvaluationService.uploadFile(subjectiveEvaluationUploadFileParam);
        return SaResult.ok("操作成功");
    }

    @ApiOperation("修改主观评价")
    @LogOperation(message = "修改主观评价")
    @PostMapping(value = "/updateSubjectiveEvaluation")
    public SaResult updateSubjectiveEvaluation(@Validated @RequestBody UpdateSubjectiveEvaluationParam updateSubjectiveEvaluationParam) {
        int result = subjectiveEvaluationService.updateSubjectiveEvaluation(updateSubjectiveEvaluationParam);
        if (result > 0) return SaResult.ok();
        return SaResult.error();
    }

    @SaCheckRole("admin")
    @ApiOperation("删除主观评价评价数据")
    @LogOperation(message = "删除主观评价评价数据")
    @PostMapping(value = "/deleteEvaluationData")
    public Result<?> deleteEvaluationData(@Validated @RequestBody DeleteEvaluationDataDTO dto) {
        subjectiveEvaluationService.deleteEvaluationData(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("主观评价获取列表")
    @PostMapping(value = "/getSubjectiveEvaluationList")
    public SaResult getSubjectiveEvaluationList(@Validated @RequestBody SubjectiveEvaluationParam subjectiveEvaluationParam) {
        return new SaResult(SaResult.CODE_SUCCESS, "成功", subjectiveEvaluationService.getSubjectiveEvaluationList(subjectiveEvaluationParam));
    }

    @ApiOperation("主观评价获取列表(多个id)")
    @PostMapping(value = "/getSubjectiveEvaluationLists")
    public SaResult getSubjectiveEvaluationLists(@Validated @RequestBody SubjectiveEvaluationsParam subjectiveEvaluationsParam) {
        Map<Integer, List<GetSubjectiveEvaluationListVO>> subjectiveEvaluationDTOS = subjectiveEvaluationService.getSubjectiveEvaluationLists(subjectiveEvaluationsParam);
        return new SaResult(SaResult.CODE_SUCCESS, "成功", subjectiveEvaluationDTOS);
    }

    @ApiOperation("主观评价生成报告")
    @LogOperation(message = "主观评价生成报告")
    @PostMapping(value = "/getSubjectiveEvaluationReports")
    public SaResult getSubjectiveEvaluationReports(@Validated @RequestBody SubjectiveEvaluationReportsParam subjectiveEvaluationReportsParam) {
        subjectiveEvaluationService.getSubjectiveEvaluationReports(subjectiveEvaluationReportsParam);
        return new SaResult(SaResult.CODE_SUCCESS, "操作成功", null);
    }

    @ApiOperation("主观评价对比关键指标数据")
    @LogOperation(message = "主观评价对比关键指标数据")
    @PostMapping(value = "/getSubjectiveDataListToCompare")
    public SaResult getSubjectiveDataListToCompare(@Validated @RequestBody SubjectiveIdDTO subjectiveIdDTO) {
        return new SaResult(SaResult.CODE_SUCCESS, "成功", subjectiveEvaluationService.getSubjectiveDataListToCompare(subjectiveIdDTO));
    }

    @ApiOperation("查询评价人列表")
    @PostMapping(value = "/queryEvaluatorNameList")
    public Result<QueryEvaluatorNameListVO> queryEvaluatorNameList(@Validated @RequestBody QueryEvaluatorNameListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, subjectiveEvaluationService.queryEvaluatorNameList(dto));
    }

    @ApiOperation("NVH性能主观评价文件主列表分页查询")
    @PostMapping(value = "/queryNVHSubjectiveFileList")
    public Result<List<QueryAttachmentListVO>> queryNVHSubjectiveFileList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,attachmentService.queryNVHSubjectiveAttachmentList(dto));
    }

    @ApiOperation("上传NVH性能主观评价文件")
    @PostMapping(value = "/uploadNVHSubjectiveFile")
    public Result<List<QueryAttachmentListVO>> uploadNVHSubjectiveFile(@Validated UploadAttachmentDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        subjectiveEvaluationService.uploadNVHSubjectiveFile(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除NVH性能主观评价文件")
    @PostMapping(value = "/deleteNVHSubjectiveFile")
    public Result<List<QueryAttachmentListVO>> deleteNVHSubjectiveFile(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());
        LambdaQueryWrapper<NVHChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(NVHChartData::getAttachId,dto.getAttachIdList());
        nvhChartDataMapper.delete(queryWrapper);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询NVH性能主观评价绘图数据")
    @PostMapping(value = "/queryNVHSubjectiveChartData")
    public Result<List<ChartDataModel>> queryNVHSubjectiveChartData(@Validated @RequestBody QueryNVHSubjectiveChartDataDTO dto) {
        List<ChartDataModel> chartDataModelList = new ArrayList<>();
        List<Integer> businessList = new ArrayList<>(Arrays.asList(20,21));
        LambdaQueryWrapper<NVHChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(NVHChartData::getCarId,dto.getCarId())
                .in(NVHChartData::getBusinessId,businessList);
        List<NVHChartData> nvhChartDataList = nvhChartDataMapper.selectList(queryWrapper);
        for (NVHChartData nvhChartData : nvhChartDataList) {
            try{
                ChartDataModel chartDataModel = (ChartDataModel)FileUtil.byteToObject(nvhChartData.getChartData());
                chartDataModelList.add(chartDataModel);
            }catch (Exception e){
                e.printStackTrace();
                throw new ToolsBusinessWarn("读取绘图数据异常");
            }
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,chartDataModelList);
    }

    //TODO 添加预览接口，minio获取IO流，再将IO流使用openoffice转换为PDF
    @ApiOperation("预览nvh文件")
    @PostMapping(value = "/previewFile")
    public void previewFile(@RequestBody PreviewParam fileParam, HttpServletResponse response) {
        Long id = fileParam.getId();
        Attachment attachmentById = attachmentService.getAttachmentById(Math.toIntExact(id));
        String fileName = attachmentById.getName();
        String type = attachmentById.getType();
        try {
            // 字符集编码
            response.setCharacterEncoding(StandardCharsets.UTF_8.name());
            //清空response中的输出流
            response.reset();
            //设置Content-Type头
            response.setContentType("application/octet-stream;charset=UTF-8");
            response.setHeader("Access-Control-Allow-Origin", "*");
            //下面这行代码必须加，否则前端获取不到Content-Disposition字段，即无法获取文件名
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            InputStream inputStream = attachmentService.downloadAttachment(attachmentById.getObjName());
            if ("docx".equals(type) || "doc".equals(type)) {
//                openOfficeUtil.wordToPDF(inputStream, response.getOutputStream());
            } else if ("xlsx".equals(type) || "xls".equals(type)) {
//                openOfficeUtil.excelToPDF(openOfficeUtil.refresh(inputStream,type), response.getOutputStream());
            } else if("pdf".equals(type)){
                BufferedInputStream bis = new BufferedInputStream(inputStream);
                BufferedOutputStream bos = new BufferedOutputStream(response.getOutputStream());
                byte[] buffer = new byte[1024];
                int length;
                while ((length = bis.read(buffer)) != -1) {
                    bos.write(buffer, 0, length);
                }
                bis.close();
                bos.close();
            }else {
                throw new SaTokenException("文件格式错误");
            }
        } catch (IOException e) {
            throw new SaTokenException("预览失败");
        }
    }

    @ApiOperation("主观评价生成实验报告获取车辆基本信息")
    @PostMapping(value = "/getSubjectiveCarData")
    public Result<List<CarColumn>> getSubjectiveCarData() {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,carColumnService.list());
    }
}
