package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.car.TyreFileDeleteDTO;
import com.singularity.pojo.dto.car.TyreFileQueryDTO;
import com.singularity.pojo.dto.car.TyreFileUploadDTO;
import com.singularity.pojo.dto.car.TireMessageQueryDTO;
import com.singularity.pojo.entity.CarBasicParam;
import com.singularity.pojo.entity.CarChassisStructure;
import com.singularity.service.AttachmentService;
import com.singularity.service.CarBasicParamService;
import com.singularity.service.CarChassisStructureService;
import com.singularity.pojo.vo.tire.QueryTireDataFileListVO;
import com.singularity.pojo.vo.tire.QueryTireMessageVO;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Api(tags = "行驶性能-轮胎相关接口")
@RestController
@RequestMapping("api/tire")
public class TireController {
    @Resource
    private CarChassisStructureService carChassisStructureService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource
    private AttachmentService attachmentService;

    @ApiOperation("查询车辆轮胎信息")
    @PostMapping(value = "/queryTireMessage")
    public Result<QueryTireMessageVO> queryTireMessage(@RequestBody @Validated TireMessageQueryDTO dto) {
        LambdaQueryWrapper<CarChassisStructure> qw = new LambdaQueryWrapper<>();
        qw.eq(CarChassisStructure::getCarId,dto.getCarId());
        QueryTireMessageVO queryTireMessageVO = new QueryTireMessageVO();
        BeanUtil.copyProperties(carChassisStructureService.getOne(qw),queryTireMessageVO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,queryTireMessageVO);
    }

    @ApiOperation("查询车辆轮胎文件")
    @PostMapping(value = "/queryTyreFile")
    public Result<List<QueryTireDataFileListVO>> queryTyreFile(@RequestBody @Validated TyreFileQueryDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryTireDataFileList(dto));
    }

    @ApiOperation("上传车辆轮胎文件")
    @PostMapping(value = "/uploadTyreFile")
    public Result<?> uploadTyreFile(@Validated TyreFileUploadDTO dto) {
        attachmentService.uploadAttachment(
                dto.getMultipartFile(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                FileBusinessConstant.TIRE_TEST_DATA_FILE
        );

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setTireUploadStat(2);
        carBasicParam.setId(dto.getCarId());
        carBasicParamService.updateById(carBasicParam);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除车辆轮胎文件")
    @PostMapping(value = "/deleteTyreFile")
    public Result<?> deleteTyreFile(@RequestBody @Validated TyreFileDeleteDTO dto) {
        attachmentService.removeAttachmentById(dto.getAttachIdList());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
