package com.singularity.controller;

import cn.dev33.satoken.secure.BCrypt;
import cn.dev33.satoken.util.SaResult;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.pojo.dto.user.UserDetailDTO;
import com.singularity.pojo.dto.user.UserListDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.User;
import com.singularity.pojo.param.user.*;
import com.singularity.service.AttachmentService;
import com.singularity.service.UserService;
import com.singularity.util.page.ResponseDTOWithPage;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@Api(tags = "用户相关")
@RestController
@RequestMapping("/user")
public class UserController {

    private final UserService userService;

    @Autowired
    public UserController(UserService userService) {
        this.userService = userService;
    }

    @Resource
    private AttachmentService attachmentService;

    @ApiOperation("获取用户列表")
    @RequestMapping(method = RequestMethod.POST,value = "/list")
    public SaResult getUserList(@RequestBody UserListParam userListParam){
        ResponseDTOWithPage<UserListDTO> list = userService.getUserList(userListParam);
        return new SaResult(SaResult.CODE_SUCCESS,"查询成功",list);
    }

    @ApiOperation("删除用户列表")
    @RequestMapping(method = RequestMethod.POST,value = "/DelUserList")
    public SaResult DelUserList(@RequestBody UserDelListParam userDelListParam){
        userService.delUserList(userDelListParam);
        return SaResult.ok("删除成功");
    }

    @ApiOperation("添加用户")
    @RequestMapping(method = RequestMethod.POST,value = "/insert")
    public SaResult insertUser(@RequestBody UserAddParam userAddParam){
        userAddParam.setPassword(BCrypt.hashpw(userAddParam.getPassword(), BCrypt.gensalt(15)));
        userService.insert(userAddParam);
        return SaResult.ok("创建成功");
    }

    @ApiOperation("修改用户")
    @RequestMapping(method = RequestMethod.POST,value = "/update")
    public SaResult updateUser(@RequestBody UserUpdateParam userUpdateParam){
        if(ObjectUtil.isNotEmpty(userUpdateParam.getProfileFile())){
            Attachment attachment = attachmentService.uploadAttachment(
                    userUpdateParam.getProfileFile(),
                    "car_info",
                    null);
            userUpdateParam.setProfilePath(attachment.getPath());
        }
        userService.update(userUpdateParam);
        return SaResult.ok("修改成功");
    }

    @ApiOperation("根据id获取用户个人信息")
    @RequestMapping(method = RequestMethod.GET,value = "/detail/{id}")
    public SaResult getInfo(@PathVariable("id") Integer id) {
        UserDetailDTO info = userService.getUserInfo(id);
        return new SaResult(SaResult.CODE_SUCCESS,"查询成功",info);
    }

    @ApiOperation("重置密码")
    @RequestMapping(method = RequestMethod.POST,value = "/updatePsw")
    public SaResult updatePsw(@RequestBody UpdatePswParam updatePswParam){
        Boolean result = userService.updatePsw(updatePswParam);
        if (result){
            return SaResult.ok("重置成功");
        }
        return SaResult.error("重置失败");
    }

    @ApiOperation("分配角色")
    @RequestMapping(method = RequestMethod.POST, value = "/distribute")
    public SaResult distributeRole(@RequestBody DistributeRoleParam distributeRoleParam) {
        if (distributeRoleParam.getDepartmentId() != null) {
            UserUpdateParam param = new UserUpdateParam();
            param.setId(distributeRoleParam.getUserId());
            param.setDepartmentId(distributeRoleParam.getDepartmentId());
            User user = BeanUtil.copyProperties(param, User.class);
            Boolean insert = userService.update(user);
            if (!insert) {
                return SaResult.error("分配失败");
            }
        }
        Boolean result = userService.distributeRole(distributeRoleParam);
        if (result){
            return SaResult.ok("分配成功");
        }
        return SaResult.error("分配失败");
    }

    @ApiOperation("根据id获取用户基本信息")
    @PostMapping("/getUserMessage")
    public Result<UserDetailDTO> getUserMessage(@RequestBody @Validated User user){
        if (user.getId() == null){
            throw new ToolsBusinessWarn("id不可为空");
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,userService.getUserInfo(user.getId()));
    }
}
