package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.util.SaResult;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.dto.vehicle.VehicleInfoDTO;
import com.singularity.pojo.dto.kc.CarDataDTO;
import com.singularity.pojo.dto.kc.CarFileDTO;
import com.singularity.pojo.param.kc.CarDataParam;
import com.singularity.service.VehicleInfoService;
import com.singularity.util.page.ResponseDTOWithPage;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@RestController
@Api(tags = "车辆信息")
@RequestMapping(value = "/api/vehicleInfo")
public class VehicleInfoController {

    @Resource
    private VehicleInfoService vehicleInfoService;

    @ApiOperation("获取车辆数据列表")
    @PostMapping(value = "/getCarDataList")
    public SaResult getCarDataList(@Validated @RequestBody CarDataParam carDataParam) {
        ResponseDTOWithPage<CarDataDTO> list = vehicleInfoService.getCarDataList(carDataParam);
        return new SaResult(SaResult.CODE_SUCCESS, "操作成功", list);
    }

    @SaCheckRole("admin")
    @ApiOperation("车辆删除")
    @LogOperation(message = "车辆信息删除")
    @DeleteMapping(value = "/deleteCarData")
    public Result<?> deleteCarData(@Validated @RequestBody List<CarDataParam> carDataParams) {
        vehicleInfoService.deleteCarData(carDataParams);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("设置车辆试验信息")
    @PostMapping(value = "/setCarData")
    public Result<?> setCarData(@Validated @RequestBody VehicleInfoDTO vehicleInfoDTO) {
        vehicleInfoService.setCarData(vehicleInfoDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("新增车辆信息")
    @PostMapping(value = "/saveCarData")
    public Result<?> saveCarData(@Validated @RequestBody CarFileDTO carFileDTO) {
        vehicleInfoService.saveCarData(carFileDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
