package com.singularity.controller;


import com.singularity.pojo.dto.vehicle.CompareSineFitCharacteristicDTO;
import com.singularity.pojo.dto.vehicle.CompareVehicleInfoDTO;
import com.singularity.pojo.dto.vehicle.VehicleInformationSearchDTO;
import com.singularity.pojo.entity.VehicleInformation;
import com.singularity.service.VehicleInformationService;
import com.singularity.pojo.vo.vehicle.CompareSineFitCharacteristicVO;
import com.singularity.pojo.vo.vehicle.CompareVehicleInfoVO;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 车辆信息 前端控制器
 * </p>
 *
 * @author winsky
 * @since 2024-03-29
 */
@RestController
@Slf4j
@Api(tags = "质心及转动惯量之车辆信息相关接口")
@RequestMapping("/api/vehicleInformation")
public class VehicleInformationController {

    @Resource
    private VehicleInformationService vehicleInformationService;

    @ApiOperation("获取车辆数据列表")
    @PostMapping(value = "getAllData")
    public Result<List<VehicleInformation>> getAllData(@Validated @RequestBody VehicleInformationSearchDTO vehicleInformationSearchDTO){
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,vehicleInformationService.getAllData(vehicleInformationSearchDTO));
    }

    @ApiOperation("修改车辆数据列表数据")
    @PostMapping(value = "setVehicleInformationData")
    public Result<?> setVehicleInformationData(@Validated @RequestBody VehicleInformation vehicleInformation){
        vehicleInformationService.setVehicleInformationData(vehicleInformation);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("对比质心及转动惯量-正弦拟合特性")
    @PostMapping(value = "compareSineFitCharacteristic")
    public Result<CompareSineFitCharacteristicVO> compareSineFitCharacteristic(@Validated @RequestBody CompareSineFitCharacteristicDTO dto){
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,vehicleInformationService.compareVehicleEvaluationInfo(dto));
    }

    @ApiOperation("对比质心及转动惯量-车辆参数")
    @PostMapping(value = "compareVehicleInfo")
    public Result<CompareVehicleInfoVO> compareVehicleInfo(@Validated @RequestBody CompareVehicleInfoDTO dto){
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,vehicleInformationService.compareVehicleInfo(dto));
    }

}

