package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.pojo.dto.kc.CarExcelDTO;
import com.singularity.pojo.enumeration.CarInfoEnum;
import com.singularity.service.KCService;
import com.singularity.pojo.dto.kc.CarFileDTO;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.List;

public class KCReadFileListener implements ReadListener<CarExcelDTO> {

    private final KCService kcService;

    private final List<CarFileDTO> cachedDataList = new ArrayList<>();

    public KCReadFileListener(KCService kcService) {
        this.kcService = kcService;
    }

    @Override
    public void invoke(CarExcelDTO carExcelDTO, AnalysisContext context) {
        CarFileDTO data = new CarFileDTO();
        BeanUtils.copyProperties(carExcelDTO,data);
        data.setRearTirePressure(carExcelDTO.getRearTirePressure());
        // 枚举转换
        data.setClassification(CarInfoEnum.ClassificationEnum.getCodeByName(carExcelDTO.getClassification()));
        data.setLevel(CarInfoEnum.LevelEnum.getCodeByName(carExcelDTO.getLevel()));
        data.setCategory(CarInfoEnum.CategoryEnum.getCodeByName(carExcelDTO.getCategory()));
        data.setProductionPlace(CarInfoEnum.ProductionPlaceEnum.getCodeByName(carExcelDTO.getProductionPlace()));
        data.setEnergy(CarInfoEnum.EnergyEnum.getCodeByName(carExcelDTO.getEnergy()));
        data.setSteeringType(CarInfoEnum.SteeringTypeEnum.getCodeByName(carExcelDTO.getSteeringType()));
        data.setFrontHangingForm(CarInfoEnum.FrontHangingEnum.getCodeByName(carExcelDTO.getFrontHangingForm()));
        data.setRearHangingForm(CarInfoEnum.RearHangingEnum.getCodeByName(carExcelDTO.getRearHangingForm()));
        cachedDataList.add(data);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        kcService.saveData(cachedDataList);
    }

}
