package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.pojo.entity.SlideTestCondition;
import com.singularity.service.SlideTestConditionService;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.Map;

@Slf4j
public class SlideTestConditionListener implements ReadListener<Map<Integer, String>> {
    private final SlideTestConditionService slideTestConditionService;

    private final SlideTestCondition slideTestCondition = new SlideTestCondition();

    // 附件id
    private final Long attachId;

    // 车辆id
    private final Long carId;

    // 当前时间
    LocalDateTime localDateTime = LocalDateTime.now();

    public SlideTestConditionListener(SlideTestConditionService slideTestConditionService, Long attachId, Long carId) {
        this.slideTestConditionService = slideTestConditionService;
        this.attachId = attachId;
        this.carId = carId;
    }

    @Override
    public void invoke(Map<Integer, String> data, AnalysisContext context) {
        int row = context.readRowHolder().getRowIndex();
        slideTestCondition.setCarId(carId);
        slideTestCondition.setAttachId(attachId);
        slideTestCondition.setCreateTime(localDateTime);
        slideTestCondition.setUpdateTime(localDateTime);

        switch (row){
            case 0:
                slideTestCondition.setCurbWeight(Double.parseDouble(data.get(1)));
                slideTestCondition.setAverageAtmosphericTemp(Double.parseDouble(data.get(3)));
                break;
            case 1:
                slideTestCondition.setTestBeforeWeight(Double.parseDouble(data.get(1)));
                slideTestCondition.setAverageAtmosphericPresure(Double.parseDouble(data.get(3)));
                break;
            case 2:
                slideTestCondition.setTestAfterWeight(Double.parseDouble(data.get(1)));
                slideTestCondition.setAverageWindSpeed(Double.parseDouble(data.get(3)));
                break;
            case 3:
                slideTestCondition.setDynamometerInertiaWeight(Double.parseDouble(data.get(1)));
                slideTestCondition.setCorrectionFactor(Double.parseDouble(data.get(3)));
                break;
            case 4:
                slideTestCondition.setEffectiveWeight(Double.parseDouble(data.get(1)));
                slideTestCondition.setAirResistanceFactor(Double.parseDouble(data.get(3)));
                break;
            case 5:
                slideTestCondition.setTheoryTestWeight(Double.parseDouble(data.get(1)));
                slideTestCondition.setSpeedAnalysisInterval(Double.parseDouble(data.get(3)));
                break;
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 滑行试验条件
        slideTestConditionService.save(slideTestCondition);
    }
}
