package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.pojo.entity.SteeringChartData;
import com.singularity.service.SteeringChartDataService;
import com.singularity.util.FileUtil;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
public class SteeringChartDataListener implements ReadListener<Map<Integer, String>> {

    private SteeringChartDataService steeringChartDataService;

    private Long attachId;

    private String xLabel;

    private String yLabel;

    private String title;
    private List<Double> xData = new ArrayList<>();

    private List<Double> yData = new ArrayList<>();

    public SteeringChartDataListener(SteeringChartDataService steeringChartDataService, Long attachId) {
        this.steeringChartDataService = steeringChartDataService;
        this.attachId = attachId;
    }

    @Override
    public void invoke(Map<Integer, String> data, AnalysisContext context) {
        int currentRow = context.readRowHolder().getRowIndex() + 1;
        if(currentRow == 1){
            xLabel = data.get(1);
            return;
        }
        if(currentRow == 2){
            yLabel = data.get(1);
            return;
        }
        if(currentRow == 3){
            title = data.get(1);
            return;
        }
        xData.add(Double.parseDouble(data.get(0)));
        yData.add(Double.parseDouble(data.get(1)));
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        saveData();
        log.info("所有数据解析完成！");
    }

    /**
     * 加上存储数据库
     */
    private void saveData() {
        SteeringChartData steeringChartData = new SteeringChartData();
        ChartDataModel chartDataModel = new ChartDataModel();
        chartDataModel.setXLabel(xLabel);
        chartDataModel.setYLabel(yLabel);
        chartDataModel.setTitle(title);
        double[][] data = new double[2][];
        data[0] = xData.stream().mapToDouble(i->i).toArray();
        data[1] = yData.stream().mapToDouble(i->i).toArray();
        chartDataModel.setData(data);
        try {
            steeringChartData.setAttachId(attachId);
            steeringChartData.setChartData(FileUtil.objectToByte(chartDataModel));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        steeringChartDataService.save(steeringChartData);
    }
}