package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.pojo.entity.SubjectiveEvaluation;
import com.singularity.pojo.param.subjectiveEvaluation.SubjectiveEvaluationExcelParam;
import com.singularity.service.SubjectiveEvaluationService;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

@Slf4j
public class SubjectiveEvaluationListener implements ReadListener<SubjectiveEvaluationExcelParam> {



    /**
     * 缓存的数据
     */
    private List<SubjectiveEvaluation> subjectiveEvaluationList = new ArrayList<>();

    /**
     * 所属车辆的id
     */
    private final Long carId;

    /**
     * 文件类型(0:商务车 1:乘用车)
     */
    private final Integer reviewType;

    private final SubjectiveEvaluationService subjectiveEvaluationService;

    private Long gradeOneSerial;

    private Long gradeTwoSerial;

    private Long serialNumber=1L;

    private Long attachId;


    public SubjectiveEvaluationListener(
            Long carId,Long attachId,Integer reviewType, SubjectiveEvaluationService subjectiveEvaluationService) {
        this.carId = carId;
        this.attachId = attachId;
        this.reviewType=reviewType;
        this.subjectiveEvaluationService = subjectiveEvaluationService;
    }


    @Override
    public void invoke(SubjectiveEvaluationExcelParam subjectiveEvaluationExcelParam, AnalysisContext analysisContext) {
        // 构造对象
        SubjectiveEvaluation subjectiveEvaluation = SubjectiveEvaluation.builder().
                carId(carId)
                .attachId(attachId)
                .reviewType(reviewType)
                .modelRatings(subjectiveEvaluationExcelParam.getModelRatings())
                .modelReviews(subjectiveEvaluationExcelParam.getModelReviews())
                .remark(subjectiveEvaluationExcelParam.getRemark())
                .build();

        //  第一个单元格不为空为1级标题
        if(subjectiveEvaluationExcelParam.getGradeOneName()!=null){
            subjectiveEvaluation.setTestName(subjectiveEvaluationExcelParam.getGradeOneName());
            subjectiveEvaluation.setGrade(1L);
            subjectiveEvaluation.setSerialNumber(serialNumber);
            gradeOneSerial=serialNumber++;
            gradeTwoSerial=null;
            //
        }else if(subjectiveEvaluationExcelParam.getGradeThreeName()!=null){
            subjectiveEvaluation.setTestName(subjectiveEvaluationExcelParam.getGradeThreeName());
            subjectiveEvaluation.setGrade(3L);
            subjectiveEvaluation.setSerialNumber(serialNumber++);
            if(gradeTwoSerial==null)
                subjectiveEvaluation.setBelongSerial(gradeOneSerial);
            else
                subjectiveEvaluation.setBelongSerial(gradeTwoSerial);


        }else {
            subjectiveEvaluation.setTestName(subjectiveEvaluationExcelParam.getGradeTwoName());
            subjectiveEvaluation.setGrade(2L);
            subjectiveEvaluation.setSerialNumber(serialNumber);
            subjectiveEvaluation.setBelongSerial(gradeOneSerial);
            gradeTwoSerial=serialNumber++;

        }
        subjectiveEvaluationList.add(subjectiveEvaluation);

    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        saveData();
    }

    /**
     * 保存到数据库
     */
    private void saveData() {
        subjectiveEvaluationService.save(subjectiveEvaluationList);
    }


}
