package com.singularity.pojo.dto.kc;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;

@Data
@AllArgsConstructor
@NoArgsConstructor
public class CarDataDTO {

    @ApiModelProperty(value = "汽车id")
    private Long id;

    @ApiModelProperty(value = "品牌")
    private String brand;

    @ApiModelProperty(value = "车型")
    private String type;


    @ApiModelProperty(value = "级别（0：A级，1：B级，2：C级，3：D级）")
    private Integer vehicleLevel;

    @ApiModelProperty(value = "类别(0:轿车,1:SUV,2:MPV,3:跑车,4:皮卡,5:面包车,6:越野车7:载客汽车,8:载货汽车,9:牵引汽车,10:专用汽车,11自卸汽车)")
    private Integer category;

    @ApiModelProperty(value = "分类（0：物理车，1：虚拟样车）")
    private Integer classification;

    @ApiModelProperty(value = "轴距")
    private Long wheelbase;

    @ApiModelProperty(value = "前轮距")
    private Long frontGauge;

    @ApiModelProperty(value = "后轮距")
    private Long backGauge;

    @ApiModelProperty(value = "转向形式 (0:液压助力转向 1：电动助力转向)")
    private Integer steeringSystem;

    @ApiModelProperty(value = "前悬挂形式(0：麦弗逊，1：双横臂，2：板簧，3：其他)")
    private Integer frontSuspensionForm;

    @ApiModelProperty(value = "后悬挂形式（0：三连杆，1：双横臂，2：麦弗逊，3：E型四连杆，4：五连杆，5、板簧，6、三连杆（非独立）7、五连杆（非独立）8、其它 9、H 臂悬架）")
    private Integer backSuspensionForm;

    @ApiModelProperty(value = "年份")
    private Long particularYear;

    @ApiModelProperty(value = "国别")
    private String country;

    @ApiModelProperty(value = "能源(0：汽油，1：柴油，2：油电混合，3：纯电动，4：插电式混动，5：增程式，6：氢燃料，7：燃料电池，8：天然气，9：甲醇，10：汽油+48V轻混系统，11：汽油+24V轻混系统)")
    private Integer energy;

    @ApiModelProperty(value = "试验类型")
    private String experimentType;

    @ApiModelProperty(value = "VIN")
    private String VIN;

    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date updatedAt;

    @ApiModelProperty(value = "KC状态（KC数据文件上传状态（0：未上传，1：上传中，2：已上传））")
    private Integer kcStat;

    @ApiModelProperty(value = "NVH状态（nvh数据文件上传状态（0：未上传，1：上传中，2：已上传））")
    private Integer nvhStat;

    // 主观评价
    @ApiModelProperty(value = "商用车行驶性能主观评价上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer commercialUploadStat;

    @ApiModelProperty(value = "乘用车行驶性能主观评价上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer passengerUploadStat;

    @ApiModelProperty(value = "乘用车商品性主观评价上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer commodityUploadStat;

    @ApiModelProperty(value = "客车商品性主观评价上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer busUploadStat;

    @ApiModelProperty(value = "货车商品性主观评价上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer truckUploadStat;

    @ApiModelProperty(value = "客车商品性主观评价上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer nvhSubjectiveStat;

    // 行驶性能状态
    @ApiModelProperty(value = "行驶性能轮胎上传状态上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer tireUploadStat;

    @ApiModelProperty(value = "行驶性能减震器上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer damperUploadStat;

    @ApiModelProperty(value = "行驶性能衬套上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer bushingUploadStat;

    @ApiModelProperty(value = "行驶性能质心及转动惯量上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer inertiaUploadStat;

    @ApiModelProperty(value = "行驶性能平顺性上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer rideUploadStat;

    @ApiModelProperty(value = "行驶性能滑行上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer coastUploadStat;

    @ApiModelProperty(value = "行驶性能操纵稳定性上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer handingUploadStat;

    @ApiModelProperty(value = "行驶性能转向上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer steeringUploadStat;

    @ApiModelProperty(value = "行驶性能制动性上传状态（0：未上传，1：上传中，2：已上传）")
    private Integer brakingUploadStat;

    @ApiModelProperty(value = "前制动形式", dataType = "Integer")
    private Integer frontRetardation;

    @ApiModelProperty(value = "后制动形式", dataType = "Integer")
    private Integer rearRetardation;

    @ApiModelProperty(value = "形式性能轮胎", dataType = "Integer")
    private Integer springUploadStat;
}
