package com.singularity.pojo.excel;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import lombok.Data;
import lombok.extern.log4j.Log4j2;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author 平
 * @description excel监听器
 */
@Log4j2
@Data
public class BaseExcelListener extends AnalysisEventListener<Map<Integer, String>> {

    private final BaseChartResult baseResult = new BaseChartResult();

    //行数
    private Long rowNum = 1L;

    //参数名称
    private String argName;

    //参数坐标范围
    private IndexRange indexOfArg;

    @Override
    public void invoke(Map<Integer, String> integerStringMap, AnalysisContext analysisContext) {
//        log.info("第{}行数据:{}", rowNum, JSONUtil.toJsonStr(integerStringMap));
        int startCol = indexOfArg.getStartCol();
        int endCol = indexOfArg.getEndCol();
        List<String> values = new ArrayList<>();
        while (startCol <= endCol) {
            values.add(integerStringMap.get(startCol - 1));
            startCol++;
        }
        switch (argName) {
            case "x":
                baseResult.getX().addAll(BeanUtil.copyToList(values, Double.class));
                break;
            case "y":
                baseResult.getY().addAll(BeanUtil.copyToList(values, Double.class));
                break;
            case "z":
                baseResult.getZ().add(BeanUtil.copyToList(values, Double.class));
                break;
            case "labelOfX":
                baseResult.setLabelOfX(values.get(0));
                break;
            case "labelOfY":
                baseResult.setLabelOfY(values.get(0));
                break;
            case "labelOfZ":
                baseResult.setLabelOfZ(values.get(0));
                break;
            case "title":
                baseResult.setTitle(values.get(0));
                break;
            case "xUnit":
                baseResult.setXUnit(values.get(0));
                break;
            case "yUnit":
                baseResult.setYUnit(values.get(0));
                break;
            default:
                baseResult.getExtraParam().put(argName, values);
        }
        rowNum++;
    }


    @Override
    public boolean hasNext(AnalysisContext context) {
        boolean hasNext = rowNum <= (indexOfArg.getEndRow() - indexOfArg.getStartRow() + 1);
        if (!hasNext) {
            rowNum = 1L;
        }
        return hasNext;
    }


    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        log.info("所有数据解析完成");
    }

}
