package com.singularity.service;

import com.singularity.pojo.dto.experiment.ExperimentFileDTO;
import com.singularity.pojo.dto.file.DeleteNFileDTO;
import com.singularity.pojo.dto.kc.*;
import com.singularity.pojo.param.kc.*;
import com.singularity.pojo.vo.car.CarBasicParamVO;
import com.singularity.pojo.vo.car.CarBodyParamVO;
import com.singularity.pojo.vo.car.CarChassisStructureVO;
import com.singularity.pojo.vo.car.SelectCarEnergyParamVO;
import com.singularity.pojo.vo.kc.CompareKCEvaluationInfoVO;
import com.singularity.pojo.vo.kc.QueryKCEvaluationInfoVO;
import com.singularity.pojo.vo.kc.ReadKCFileListVO;
import com.singularity.pojo.vo.kc.ReadKCFileVO;
import com.singularity.util.page.ResponseDTOWithPage;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

public interface KCService {

    /**
     * KC读取数据文件
     */
    List<ReadKCFileVO> readKCFile(ReadKCFileDTO dto);


    /**
     * 批量读取K&C数据文件
     */
    List<ReadKCFileListVO> readKCFileList(ReadKCFileListDTO dto);

    /**
     * 车辆信息主列表分页查询
     */
    ResponseDTOWithPage<CarDataDTO> getCarDataList(CarDataParam carDataParam);

    /**
     * 获取统计数据列表
     *
     * @return 车辆数据列表
     */
    CarDataStatisticsDTO getCarDataStatistics(CarDataStatisticsParam carDataStatisticsParam);

    /**
     * 获取实验条件及工况
     *
     * @param experimentConditionParam 筛选参数
     * @return 实验条件及工况
     */
    ExperimentConditionDTO getExperimentCondition(ExperimentConditionParam experimentConditionParam);

    int insertExperimentCondition(SetExperimentParamDTO setExperimentParamDTO);

    List<CarBasicParamVO> selectCarBasicParam(CarMessageIdDTO carMessageIdDTO);

    List<CarBodyParamVO> selectCarBodyParam(CarMessageIdDTO carMessageIdDTO);

    List<CarChassisStructureVO> selectCarChassisStructure(CarMessageIdDTO carMessageIdDTO);

    int addCarDataList(CarDataAddParam carDataAddParam);

    ResponseDTOWithPage<KeyIndexDTO> getKeyIndexList(KeyIndexParam keyIndexParam);

    void importFileByName(KCUploadCarFileDTO kcUploadCarFileDTO);

    void saveData(List<CarFileDTO> cachedDataList);

    void updateCarBodyParam(CarFileDTO carMessageIdDTO);

    void updateCarBasicParam(CarFileDTO carMessageIdDTO);

    void updateCarChassisStructure(CarFileDTO carMessageIdDTO);

    /**
     * 获取K&C关键指标信息
     */
    List<QueryKCEvaluationInfoVO> queryKCEvaluationInfo(QueryKCEvaluationInfoDTO dto);

    /**
     * 对比K&C关键指标信息
     */
    CompareKCEvaluationInfoVO compareKCEvaluationInfo(CompareKCEvaluationInfoDTO dto);

    /**
     * 批量删除KC附件
     */
    void deleteKCAttachmentByIdList(DeleteNFileDTO deleteNFileDTO);

    /**
     * 导出K&C关键指标信息
     * @param dto 查询条件
     * @param response 响应
     */
    void exportKCEvaluationInfo(ExportKCEvaluationInfoDTO dto, HttpServletResponse response);

    /**
     * 异步上传K&C试验数据文件
     */
    void asyncUploadKCConditions(ExperimentFileDTO experimentFileDTO);

    /**
     * 上传K&C试验数据文件
     */
    void uploadKCFileData(UploadKCDataFile uploadKCDataFile);

    /**
     * 查询车辆动力参数
     */
    List<SelectCarEnergyParamVO> selectCarEnergyParam(CarMessageIdDTO carMessageIdDTO);

    /**
     * 修改车辆动力参数
     */
    void updateCarEnergyParam(CarFileDTO carMessageIdDTO);
}
