package com.singularity.service;

import com.singularity.common.vo.nvh.ReadNVHFileVO;
import com.singularity.pojo.dto.menuInfo.MenuInfoDTO;
import com.singularity.pojo.dto.nvh.*;
import com.singularity.pojo.param.nvh.NVHContentAddParam;
import com.singularity.pojo.param.nvh.NVHSelectParam;
import com.singularity.pojo.param.nvh.NVHUpdateParam;
import com.singularity.pojo.param.nvh.NVHUploadFileParam;
import com.singularity.pojo.vo.nvh.*;

import java.util.List;

public interface NVHService {

    /**
     * 查询NVH菜单树
     */
    List<GetNVHMenuTreeVO> getNVHMeuTreeList(GetNVHMenuTreeDTO dto);

    /**
     * 递归构建菜单信息 - 一次sql查询全部菜单信息
     *
     * @param result       菜单信息返回列表
     * @param allRightMenu 所有菜单信息
     * @param parentId     父级id
     * @param level        层级
     */
    void getMenuInfoDTOS(List<MenuInfoDTO> result, List<MenuInfoDTO> allRightMenu, Long parentId, Long level);


    /**
     * 添加NVH目录列表
     *
     * @param addParam 添加参数
     * @return 数据id
     */
    long addNVHContentList(NVHContentAddParam addParam);


    /**
     * NVH对比目录列表
     *
     * @param selectParam 查询参数
     * @return 查询结果
     */
    List<MenuInfoDTO> getNVHContentListForCompare(NVHSelectParam selectParam);


    void uploadNVHFile(NVHUploadFileParam dto);

    /**
     * NVH目录删除
     *
     * @param id 目录id
     * @return 删除结果
     */
    void deleteNVHContentList(Long id);

    /**
     * NVH目录更新
     *
     * @param updateParam 更新参数
     * @return 更新结果
     */
    int updateNVHContentList(NVHUpdateParam updateParam);

    /**
     * NVH文件删除
     *
     * @param id 文件id
     * @return 删除结果
     */
    int deleteNVHFile(Long id);

    /**
     * 异步批量上传文件
     */
    void asyncUploadFileList(UploadFileListDTO dto);


    /**
     * 读取NVH数据文件
     */
    List<ReadNVHFileVO> readNVHFile(ReadNVHFileDTO dto);


    /**
     * 批量读取NVH数据文件
     */
    ReadNVHFileListVO readNVHFileList(ReadNVHFileListDTO dto);

    CompareNVHSubjectiveFileVO compareNVHSubjectiveFile(CompareNVHSubjectiveFileDTO dto);

   QueryNVHDataFileListVO queryNVHDataFileList(QueryNVHDataFileListDTO dto);

    CompareNVHObjectiveFileVO compareNVHObjectiveFile(CompareNVHObjectiveFileDTO dto);
}
