package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.car.inertia.ComputeInertiaData;
import com.github.pagehelper.PageHelper;
import com.mathworks.toolbox.javabuilder.MWCellArray;
import com.singularity.dao.AttachmentMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.dto.attachment.DownloadZipFileDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryResourceFileListDTO;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.barking.QueryBarkingDataFileListDTO;
import com.singularity.pojo.dto.bushing.QueryBushingDataFileListDTO;
import com.singularity.pojo.dto.car.TyreFileQueryDTO;
import com.singularity.pojo.dto.damper.QueryDamperAttachmentDTO;
import com.singularity.pojo.dto.experiment.ExperimentFileDTO;
import com.singularity.pojo.dto.handing.QueryHandingFileListDTO;
import com.singularity.pojo.dto.handing.UploadReSourceFileDTO;
import com.singularity.pojo.dto.kc.CarExperimentParamListQueryDTO;
import com.singularity.pojo.dto.kc.KCFileSearch;
import com.singularity.pojo.dto.ride.QueryRideFileListDTO;
import com.singularity.pojo.dto.steering.QuerySteeringDataFileListDTO;
import com.singularity.pojo.dto.template.QueryTempAttachmentListDTO;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.enumeration.ExperimentEnum;
import com.singularity.pojo.enumeration.HandingEnum;
import com.singularity.pojo.vo.attachment.QueryResourceFileListVO;
import com.singularity.pojo.vo.attachment.SelectAttachmentByFileNameVO;
import com.singularity.pojo.vo.attachment.UploadAttachmentVO;
import com.singularity.pojo.vo.barking.QueryBarkingDataFileListVO;
import com.singularity.pojo.vo.bushing.QueryBushingDataFileListVO;
import com.singularity.pojo.vo.car.CarExperimentParamListQueryVO;
import com.singularity.pojo.vo.damper.QueryDamperDataFileListVO;
import com.singularity.pojo.vo.hading.QueryHandingFileListVO;
import com.singularity.pojo.vo.ride.QueryRideFileListVO;
import com.singularity.pojo.vo.steering.QuerySteeringDataFileListVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.pojo.vo.template.QueryTempAttachmentListVO;
import com.singularity.pojo.vo.tire.QueryTireDataFileListVO;
import com.singularity.service.*;
import com.singularity.util.FileUtil;
import com.singularity.util.page.PageUtil;
import com.singularity.util.page.ResponseDTOWithPage;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.*;
import java.nio.charset.Charset;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

@Slf4j
@Service
public class AttachmentServiceImpl extends ServiceImpl<AttachmentMapper, Attachment> implements AttachmentService {

    @Resource
    private MinioService minioService;

    @Resource
    private AttachmentMapper attachmentMapper;

    @Value("${file.path}")
    private String filePath;

    @Resource
    private VehicleInformationService vehicleInformationService;

    @Resource
    private SineFitCharacteristicService sineFitCharacteristicService;

    @Resource(name = "threadPool")
    private ThreadPoolTaskExecutor threadPool;

    @Resource
    private TaskQueueService taskQueueService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource
    private ExperimentService experimentService;

    @Override
    public Attachment uploadAttachment(MultipartFile multipartFile, String ownerType, Long ownerId) {
        UploadAttachmentVO uploadAttachmentVO = minioService.storeFile(multipartFile);
        Attachment attachment = BeanUtil.copyProperties(uploadAttachmentVO, Attachment.class);
        attachment.setOwnerType(ownerType);
        attachment.setOwnerId(ownerId);
        attachmentMapper.insert(attachment);
        return attachment;
    }

    @Override
    public Attachment uploadAttachment(MultipartFile multipartFile, String ownerType, Long ownerId, int businessType) {
        UploadAttachmentVO uploadAttachmentVO = minioService.storeFile(multipartFile);
        Attachment attachment = BeanUtil.copyProperties(uploadAttachmentVO, Attachment.class);
        attachment.setOwnerType(ownerType);
        attachment.setOwnerId(ownerId);
        attachment.setBusinessType(businessType);
        attachmentMapper.insert(attachment);
        return attachment;
    }

    @Override
    public Attachment uploadAttachment(MultipartFile multipartFile, String ownerType, Long ownerId, int businessType, int operId) {
        this.checkRepeatFileName(multipartFile.getOriginalFilename(),ownerId,businessType,ownerType);
        Attachment attachment = minioService.uploadFile(multipartFile);
        attachment.setOwnerType(ownerType);
        attachment.setOwnerId(ownerId);
        attachment.setBusinessType(businessType);
        attachment.setOperId(operId);
        attachmentMapper.insert(attachment);
        return attachment;
    }

    @Override
    @Transactional
    @SneakyThrows
    public void uploadZipAttachment(MultipartFile multipartFile, String ownerType, Long ownerId, int businessType, int operId) {
        InputStream inputStream = multipartFile.getInputStream();
        ZipInputStream zipInputStream = new ZipInputStream(inputStream, Charset.forName("GBK"));
        ZipEntry zipEntry = zipInputStream.getNextEntry();
        while (zipEntry != null) {
            if (!zipEntry.isDirectory()) {
                // 获取文件名称
                String[] currentFilePath = zipEntry.getName().split("/");
                String fileName = currentFilePath[currentFilePath.length - 1];
                // 判断文件格式是否为 .docx,.doc,.pdf 格式
                if(fileName.contains(".docx") || fileName.contains(".doc") || fileName.contains(".pdf")){
                    InputStream zipEntryStream = new ByteArrayInputStream(IOUtils.toByteArray(zipInputStream));
                    this.uploadAttachment(
                            zipEntryStream,
                            fileName,
                            zipEntryStream.available(),
                            AttachmentConstant.OWNER_TYPE_CAR,
                            ownerId,
                            businessType,
                            operId
                    );
                }

            }
            zipInputStream.closeEntry();
            zipEntry = zipInputStream.getNextEntry();
        }
        zipInputStream.close();
        inputStream.close();
    }

    @Override
    @Transactional
    @SneakyThrows
    public void uploadHandingZipAttachment(UploadReSourceFileDTO dto) {
        List<ExperimentInfo> experimentInfoList = new ArrayList<>();
        InputStream inputStream = dto.getMultipartFile().getInputStream();
        ZipInputStream zipInputStream = new ZipInputStream(inputStream, Charset.forName("GBK"));
        ZipEntry zipEntry = zipInputStream.getNextEntry();
        while (zipEntry != null) {
            if (!zipEntry.isDirectory()) {
                // 获取文件名称
                String[] currentFilePath = zipEntry.getName().split("/");
                String fileName = currentFilePath[currentFilePath.length - 1];
                Integer businessId =  HandingEnum.getCodeByName(currentFilePath[1]);
                if(ObjectUtil.isNull(businessId))
                    throw new ToolsBusinessWarn("试验类别读取错误");
                InputStream zipEntryStream = new ByteArrayInputStream(IOUtils.toByteArray(zipInputStream));
                Attachment attachment = this.uploadAttachment(
                        zipEntryStream,
                        fileName,
                        zipEntryStream.available(),
                        AttachmentConstant.OWNER_TYPE_CAR,
                        dto.getOwnerId(),
                        businessId,
                        dto.getOperId()
                );
                // 保存试验场景信息
                ExperimentInfo experimentInfo = this.readExperimentByFilePath(currentFilePath[2]);
                experimentInfo.setAttachId(attachment.getId());
                experimentInfoList.add(experimentInfo);
            }
            zipInputStream.closeEntry();
            zipEntry = zipInputStream.getNextEntry();
        }
        zipInputStream.close();
        inputStream.close();
        experimentService.saveBatch(experimentInfoList);
    }

    private ExperimentInfo readExperimentByFilePath(String filePath){
        ExperimentInfo experimentInfo = ExperimentInfo.builder().build();
        String[] filePathList = filePath.split("_");
        for (int i = 0; i < filePathList.length; i++) {
            if("数据类型".equals(filePathList[i])){
                Integer dataType = ExperimentEnum.DataTypeEnum.getCodeByName(filePathList[i + 1]);
                if(ObjectUtil.isNull(dataType))
                    throw new ToolsBusinessWarn("数据类型读取错误");
                experimentInfo.setDataType(dataType);
            } else if ("转向".equals(filePathList[i])) {
                Integer steering = ExperimentEnum.SteeringEnum.getCodeByName(filePathList[i + 1]);
                if(ObjectUtil.isNull(steering))
                    throw new ToolsBusinessWarn("转向读取错误");
                experimentInfo.setSteering(ExperimentEnum.SteeringEnum.getCodeByName(filePathList[i + 1]));
            }else if ("试验车速".equals(filePathList[i])) {
                Integer testSpeed =  ExperimentEnum.TestSpeedEnum.getCodeByName(filePathList[i + 1]);
                if(ObjectUtil.isNull(testSpeed))
                    throw new ToolsBusinessWarn("试验车速读取错误");
                experimentInfo.setTestSpeed(testSpeed);
            }
        }
        return experimentInfo;
    }

    /**
     * 批量上传原始数据文件
     */
    @Override
    @Transactional
    @SneakyThrows
    public void uploadResourceFileList(MultipartFile multipartFile, String ownerType, Long ownerId, int businessType,int operId) {
        List<ExperimentInfo> experimentInfoList = new ArrayList<>();
        InputStream inputStream = multipartFile.getInputStream();
        ZipInputStream zipInputStream = new ZipInputStream(inputStream, Charset.forName("GBK"));
        ZipEntry zipEntry = zipInputStream.getNextEntry();
        while (zipEntry != null) {
            if (!zipEntry.isDirectory()) {
                // 获取文件名称
                String[] currentFilePath = zipEntry.getName().split("/");
                String fileName = currentFilePath[currentFilePath.length - 1];
                InputStream zipEntryStream = new ByteArrayInputStream(IOUtils.toByteArray(zipInputStream));
                Attachment attachment = this.uploadAttachment(
                        zipEntryStream,
                        fileName,
                        zipEntry.getSize(),
                        ownerType,
                        ownerId,
                        businessType,
                        operId
                );
                ExperimentInfo experimentInfo = this.readExperimentByFilePath(currentFilePath[1]);
                experimentInfo.setAttachId(attachment.getId());
                experimentInfoList.add(experimentInfo);
            }
            zipInputStream.closeEntry();
            zipEntry = zipInputStream.getNextEntry();
        }
        zipInputStream.close();
        inputStream.close();
        // 保存试验场景信息 
        experimentService.saveBatch(experimentInfoList);
    }

    /**
     * 批量文件上传
     */
    @Override
    public void uploadAttachmentList(List<MultipartFile> multipartFileList, int businessType) {
        List<Attachment> attachmentList = minioService.storeFiles(multipartFileList);
        attachmentList.forEach(attachment -> attachment.setBusinessType(businessType));
        attachmentMapper.insertBatch(attachmentList);
    }

    @Override
    public List<Attachment> uploadAttachmentList(List<MultipartFile> multipartFileList, String ownerType, Long ownerId, int businessType) {
        List<Attachment> attachmentList = minioService.storeFiles(multipartFileList);
        attachmentList.forEach(attachment -> {
            attachment.setOwnerType(ownerType);
            attachment.setOwnerId(ownerId);
            attachment.setBusinessType(businessType);
        });
        attachmentMapper.insertBatch(attachmentList);
        return attachmentList;
    }

    @Override
    public void uploadAttachmentList(List<MultipartFile> multipartFileList, String ownerType, Long ownerId, int businessType,int operId) {
        List<String> fileNameList = new ArrayList<>();
        multipartFileList.forEach(multipartFile->fileNameList.add(multipartFile.getOriginalFilename()));
        // 文件名称验重
        this.checkRepeatFileName(fileNameList,ownerId,businessType,ownerType);

        List<Attachment> attachmentList = minioService.storeFiles(multipartFileList);
        attachmentList.forEach(attachment -> {
            attachment.setOwnerType(ownerType);
            attachment.setOwnerId(ownerId);
            attachment.setBusinessType(businessType);
            attachment.setOperId(operId);
        });
        attachmentMapper.insertBatch(attachmentList);
    }

    @Override
    public Attachment uploadAttachment(InputStream in, String fileName, long size, String ownerType, Long ownerId, int businessType) {
        UploadAttachmentVO uploadAttachmentVO = minioService.storeFile(in, fileName, size);
        Attachment attachment = BeanUtil.copyProperties(uploadAttachmentVO, Attachment.class);
        attachment.setOwnerType(ownerType);
        attachment.setOwnerId(ownerId);
        attachment.setBusinessType(businessType);
        attachmentMapper.insert(attachment);
        return attachment;
    }

    @Override
    public Attachment uploadAttachment(InputStream in, String fileName, long size, String ownerType, Long ownerId, int businessType, int operId) {
        UploadAttachmentVO uploadAttachmentVO = minioService.storeFile(in, fileName, size);
        Attachment attachment = BeanUtil.copyProperties(uploadAttachmentVO, Attachment.class);
        attachment.setOwnerType(ownerType);
        attachment.setOwnerId(ownerId);
        attachment.setBusinessType(businessType);
        attachment.setOperId(operId);
        attachmentMapper.insert(attachment);
        return attachment;
    }


    @Override
    public List<Attachment> getAttachment(String ownerType, Long ownerId) {
        LambdaQueryWrapper<Attachment> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Attachment::getOwnerType, ownerType);
        queryWrapper.eq(Attachment::getOwnerId, ownerId);
        return attachmentMapper.selectList(queryWrapper);
    }

    @Override
    public List<Attachment> getAttachment(String ownerType, Long ownerId, int businessType) {
        LambdaQueryWrapper<Attachment> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Attachment::getOwnerType, ownerType)
                .eq(Attachment::getOwnerId, ownerId)
                .eq(Attachment::getBusinessType, businessType);
        return attachmentMapper.selectList(queryWrapper);
    }

    @Override
    public List<Attachment> getAttachment(Integer businessType) {
        LambdaQueryWrapper<Attachment> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Attachment::getBusinessType, businessType);
        return attachmentMapper.selectList(queryWrapper);
    }

    @Override
    public List<Attachment> getAttachment(List<Integer> businessTypeList) {
        LambdaQueryWrapper<Attachment> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(Attachment::getBusinessType, businessTypeList);
        return attachmentMapper.selectList(queryWrapper);
    }


    @Override
    public Attachment getAttachmentById(Integer id) {
        LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Attachment::getId, id);
        return this.getOne(queryWrapper);
    }

    @Override
    public InputStream downloadAttachment(String objName) {
        return minioService.getFile(objName);
    }

    @Override
    public void removeAttachment(String ownerType, Long ownerId) {
        List<Attachment> attachmentList = this.getAttachment(ownerType, ownerId);
        List<Long> attachmentIds = CollStreamUtil.toList(attachmentList, Attachment::getId);
        List<String> attachmentObjNames = CollStreamUtil.toList(attachmentList, Attachment::getObjName);
        minioService.removeList(attachmentObjNames);
        this.removeByIds(attachmentIds);
    }

    @Override
    public int removeAttachmentById(Long attachmentId) {
        Attachment attachment = attachmentMapper.selectById(attachmentId);
        minioService.remove(attachment.getObjName());
        return attachmentMapper.deleteById(attachmentId);
    }

    @Override
    public int removeAttachmentById(List<Long> attachmentIds) {
        if (attachmentIds.size() == 0) {
            return 0;
        }
        List<Attachment> attachments = attachmentMapper.selectBatchIds(attachmentIds);
        List<String> attachmentObjNames = CollStreamUtil.toList(attachments, Attachment::getObjName);
        minioService.removeList(attachmentObjNames);
        return attachmentMapper.deleteBatchIds(attachmentIds);
    }

    @Override
    public void updateAttachment(List<Long> attachmentIds, String ownerType, Long ownerId) {
        LambdaQueryWrapper<Attachment> lqw = new LambdaQueryWrapper<>();
        lqw.in(Attachment::getId, attachmentIds);
        Attachment newAttachment = new Attachment();
        newAttachment.setOwnerType(ownerType);
        newAttachment.setOwnerId(ownerId);
        attachmentMapper.update(newAttachment, lqw);
    }


    @Override
    public ResponseDTOWithPage<SelectAttachmentByFileNameVO> selectByFileMessages(KCFileSearch kcFileSearch) {
        //分页查询
        PageUtil.startPage(kcFileSearch, true, SelectAttachmentByFileNameVO.class);
        return PageUtil.getListDTO(attachmentMapper.selectByFileMessages(kcFileSearch), kcFileSearch);
    }

    @Override
    public void asyncUpdateInertiaFile(ExperimentFileDTO experimentFileDTO) {
        // 创建队列信息
        LocalDateTime localDateTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(experimentFileDTO.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(experimentFileDTO.getCarId());
        taskQueue.setStat(1);
        taskQueue.setOperId(experimentFileDTO.getOperId());
        taskQueueService.save(taskQueue);
        experimentFileDTO.setStartTime(localDateTime);
        experimentFileDTO.setTaskId(taskQueue.getId());

        //将文件保存到本地
        try {
            MultipartFile multipartFile = experimentFileDTO.getMultipartFile();
            experimentFileDTO.setOriginalFilename(multipartFile.getOriginalFilename());
            String uuid = UUID.randomUUID().toString().replace("-", "");
            String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
            FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
            experimentFileDTO.setLocalFilePath(fileNameAndPath);
            experimentFileDTO.setSize(multipartFile.getSize());

        } catch (IOException e) {
            e.printStackTrace();
        }

        // 异步调用任务
        threadPool.execute(() -> {
            try {
                // 核心任务
                this.updateInertiaFile(experimentFileDTO);

                // 任务执行完毕修改任务状态
                log.info("文件上传完毕");
                LocalDateTime now = LocalDateTime.now();
                Duration duration = Duration.between(experimentFileDTO.getStartTime(),now);
                LambdaUpdateWrapper<TaskQueue> updateWrapper = Wrappers.lambdaUpdate();
                updateWrapper.set(TaskQueue::getStat,2)
                        .set(TaskQueue::getConsumeTime,duration.toMillis()/1000)
                        .eq(TaskQueue::getId,experimentFileDTO.getTaskId());
                taskQueueService.update(updateWrapper);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
                // 修改任务状态
                LambdaUpdateWrapper<TaskQueue> updateWrapper = Wrappers.lambdaUpdate();
                updateWrapper.set(TaskQueue::getStat,3)
                        .eq(TaskQueue::getId,taskQueue.getId());
                taskQueueService.update(updateWrapper);
            }finally {
                FileUtil.deleteFile(experimentFileDTO.getLocalFilePath());
                if (!experimentFileDTO.getLocalFilePathList().isEmpty()){
                    for (String path : experimentFileDTO.getLocalFilePathList()) {
                        FileUtil.deleteFile(path);
                    }
                }
            }

            // 修改文件上传状态
            CarBasicParam carBasicParam = new CarBasicParam();
            carBasicParam.setInertiaUploadStat(2);
            carBasicParam.setId(experimentFileDTO.getOwnerId());
            carBasicParamService.updateById(carBasicParam);
        });

    }

    @Transactional
    public void updateInertiaFile(ExperimentFileDTO experimentFileDTO) throws Exception{
        // 1、获取文件格式
        File file = new File(experimentFileDTO.getLocalFilePath());
        String originalFilename = file.getName();
        String[] splitFileName = originalFilename.split("\\.");
        // 文件格式
        String fileType = splitFileName[splitFileName.length - 1];
        // 2.判断是否为zip文件
        if("zip".equals(fileType)){
            InputStream inputStream = new FileInputStream(file);
            ZipInputStream zipInput = new ZipInputStream(inputStream, Charset.forName("GBK"));
            ZipEntry entry;
            List<String> paths = new ArrayList<>();
            while ((entry = zipInput.getNextEntry()) != null) {
                if (!entry.isDirectory() && "tab".equals(entry.getName().split("\\.")[1])) {
                    String fileName = entry.getName();
                    fileName = fileName.split("/")[1];
                    String newPath = filePath + fileName.replace(".tab", ".txt");
                    File outFile = new File(newPath);

                    try (
                            FileOutputStream fileOutputStream = new FileOutputStream(outFile);
                    ){
                        byte[] buffer = new byte[1024];
                        int length;
                        while ((length = zipInput.read(buffer)) > 0) {
                            fileOutputStream.write(buffer, 0, length);
                        }
                        paths.add(newPath);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }

            inputStream.close();
            zipInput.close();

            experimentFileDTO.setLocalFilePathList(paths);
            if (paths.size() != 3) {
                throw new ToolsBusinessWarn("压缩包内文件数量错误");
            }

            Collections.sort(paths, (o1, o2) -> 02 - 01);
            ComputeInertiaData computeInertiaData = new ComputeInertiaData();
            Object[] results = computeInertiaData.getVIMData(2, paths.get(0), paths.get(1), paths.get(2));
            FileInputStream saveFileInputStream = new FileInputStream(file);
            // 将本地文件上传到minio
            Attachment attachment = this.uploadAttachment(
                    saveFileInputStream,
                    experimentFileDTO.getOriginalFilename(),
                    experimentFileDTO.getSize(),
                    experimentFileDTO.getOwnerType(),
                    experimentFileDTO.getOwnerId(),
                    experimentFileDTO.getBusinessType());

            saveFileInputStream.close();


            // 3、调用matlab函数生成关键指标
            // 关键指标集合
            List<VehicleInformation> vehicleInformationList = new ArrayList<>();
            List<SineFitCharacteristic> sineFitCharacteristicList = new ArrayList<>();
            MWCellArray mwCellArray = (MWCellArray) results[0];
            for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
                Object[] data = mwCellArray.getCell(new int[]{1, i + 1}).toArray();
                VehicleInformation vehicleInformation = new VehicleInformation();
                vehicleInformation.setCarId(Math.toIntExact(experimentFileDTO.getOwnerId()));
                vehicleInformation.setAttachId(Math.toIntExact(attachment.getId()));
                vehicleInformation.setName(data[0].toString());
                vehicleInformation.setValue(data[1].toString());
                vehicleInformationList.add(vehicleInformation);
            }

            mwCellArray = (MWCellArray) results[1];
            for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
                Object[] data = mwCellArray.getCell(new int[]{1, i + 1}).toArray();
                SineFitCharacteristic sineFitCharacteristic = new SineFitCharacteristic();
                sineFitCharacteristic.setCarId(Math.toIntExact(experimentFileDTO.getOwnerId()));
                sineFitCharacteristic.setAttachId(Math.toIntExact(attachment.getId()));
                sineFitCharacteristic.setName(data[0].toString());
                sineFitCharacteristic.setFreq(data[1].toString());
                sineFitCharacteristic.setAmp(data[2].toString());
                sineFitCharacteristic.setAccel(data[3].toString());
                sineFitCharacteristic.setOffset(data[4].toString());
                sineFitCharacteristic.setAmplitude(data[5].toString());
                sineFitCharacteristic.setPhase(data[6].toString());
                sineFitCharacteristic.setTotal(data[7].toString());
                sineFitCharacteristic.setCgValue(data[8].toString());
                sineFitCharacteristicList.add(sineFitCharacteristic);
            }

            // 4、将关键指标保存到数据库
            vehicleInformationService.saveBatch(vehicleInformationList);
            sineFitCharacteristicService.saveBatch(sineFitCharacteristicList);
        }else {
            throw new ToolsBusinessWarn("应该为zip文件");
        }

    }

    /**
     * 查询车辆试验参数列表
     */
    @Override
    public List<CarExperimentParamListQueryVO> queryCarExperimentParamList(CarExperimentParamListQueryDTO dto) {
        return attachmentMapper.queryCarExperimentParamList(dto);
    }

    /**
     * 查询操作稳定性数据文件列表
     */
    @Override
    public List<QueryHandingFileListVO> queryHandingFileList(QueryHandingFileListDTO dto) {
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryHandingFileList(dto);
    }

    /**
     * 文件列表主查询
     */
    @Override
    public List<QueryAttachmentListVO> queryAttachmentList(QueryAttachmentListDTO dto) {
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryAttachmentList(dto);
    }

    @Override
    public List<QueryAttachmentListVO> queryNVHSubjectiveAttachmentList(QueryAttachmentListDTO dto) {
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryNVHSubjectiveAttachmentList(dto);
    }

    @Override
    public Attachment uploadAttachment(UploadAttachmentDTO dto) {
        MultipartFile multipartFile = dto.getMultipartFile();
        // 文件名验重
        this.checkRepeatFileName(
                dto.getMultipartFile().getOriginalFilename(),
                dto.getOwnerId(),
                dto.getBusinessType(),
                dto.getOwnerType()
        );
        UploadAttachmentVO uploadAttachmentVO = minioService.storeFile(multipartFile);
        Attachment attachment = BeanUtil.copyProperties(uploadAttachmentVO, Attachment.class);
        attachment.setOwnerType(dto.getOwnerType());
        attachment.setOwnerId(dto.getOwnerId());
        attachment.setBusinessType(dto.getBusinessType());
        attachment.setSize(multipartFile.getSize());
        attachment.setOperId(dto.getOperId());
        attachmentMapper.insert(attachment);
        return attachment;
    }

    @Override
    public List<QueryDamperDataFileListVO> queryDamperFile(QueryDamperAttachmentDTO dto) {
        PageHelper.startPage(dto.getPageNum(),dto.getPageSize());
        return attachmentMapper.queryDamperDataFileList(dto);
    }

    /**
     * 模板文件列表主查询
     */
    @Override
    public List<QueryTempAttachmentListVO> queryTempAttachmentList(QueryTempAttachmentListDTO dto){
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryTempAttachmentList(dto);
    }

    @Override
    public List<QueryBushingDataFileListVO> queryBushingDataFileList(QueryBushingDataFileListDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryBushingDataFileList(dto);
    }

    @Override
    public List<QueryBarkingDataFileListVO> queryBarkingDataFileList(QueryBarkingDataFileListDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryBarkingDataFileList(dto);
    }

    @Override
    public List<QueryTireDataFileListVO> queryTireDataFileList(TyreFileQueryDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryTireDataFileList(dto);
    }

    @Override
    public List<QuerySteeringDataFileListVO> querySteeringDataFileList(QuerySteeringDataFileListDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.querySteeringDataFileList(dto);
    }

    /**
     * 文件名称验重
     */
    @Override
    public void checkRepeatFileName(String fileName, Long ownerId, Integer businessId, String ownerType) {
        // 去除所有空格
        fileName = StringUtils.deleteWhitespace(fileName);
        LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Attachment::getName,fileName)
                .eq(Attachment::getOwnerType,ownerType)
                .eq(Attachment::getOwnerId,ownerId)
                .eq(Attachment::getBusinessType,businessId);
        if(this.count(queryWrapper) > 0){
            throw new ToolsBusinessWarn("文件名称不能重复！");
        }
    }

    /**
     * 文件名称验重
     */
    @Override
    public void checkRepeatFileName(List<String> fileNameList, Long ownerId, Integer businessId, String ownerType) {
        if(fileNameList.size() > 1 && new HashSet<>(fileNameList).size() <fileNameList.size()){
            throw new ToolsBusinessWarn("压缩包中文件名称重复！");
        }
        LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(Attachment::getName,fileNameList)
                .eq(Attachment::getOwnerType,ownerType)
                .eq(Attachment::getOwnerId,ownerId)
                .eq(Attachment::getBusinessType,businessId);
        if(this.count(queryWrapper) > 0){
            throw new ToolsBusinessWarn("文件名称不能重复！");
        }
    }

    /**
     * 平顺性文件列表查询
     */
    @Override
    public ResponseDTOWithPage<QueryRideFileListVO> queryRideFileList(QueryRideFileListDTO dto) {
        //分页查询
        PageUtil.startPage(dto, true, SelectAttachmentByFileNameVO.class);
        return PageUtil.getListDTO(attachmentMapper.queryRideFileList(dto), dto);
    }

    /**
     * 查询原始文件列表
     */
    @Override
    public List<QueryResourceFileListVO> queryResourceFileList(QueryResourceFileListDTO dto) {
        PageHelper.startPage(dto.getPageNum(), dto.getPageSize());
        return attachmentMapper.queryResourceFileList(dto);
    }

    @Override
    public List<Attachment> queryAttachmentObjNameList(DownloadZipFileDTO dto) {
        return attachmentMapper.queryAttachmentObjNameList(dto);
    }

}
