package com.singularity.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.car.ride.RideMatlabFunction;
import com.mathworks.toolbox.javabuilder.MWCellArray;
import com.mathworks.toolbox.javabuilder.MWNumericArray;
import com.singularity.dao.BushingChartDataMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.dto.bushing.CompareBushingDataDTO;
import com.singularity.pojo.dto.bushing.DeleteBushingDataFileDTO;
import com.singularity.pojo.dto.bushing.QueryBushingDataDTO;
import com.singularity.pojo.dto.bushing.UploadBushingFileDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.common.entity.BushingChartData;
import com.singularity.pojo.entity.CarBasicParam;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.pojo.vo.bushing.CompareBushingDataVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.BushingChartDataService;
import com.singularity.service.CarBasicParamService;
import com.singularity.util.FileUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.FileInputStream;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class BushingChartDataServiceImpl extends ServiceImpl<BushingChartDataMapper, BushingChartData> implements BushingChartDataService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @SneakyThrows
    @Transactional
    public void uploadBushingFile(UploadBushingFileDTO dto) {
        // 判断文件名是否重复
        LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Attachment::getName,dto.getOriginalFilename())
                .eq(Attachment::getObjName,dto.getCarId())
                .eq(Attachment::getOwnerType, AttachmentConstant.OWNER_TYPE_CAR);
        if(attachmentService.count(queryWrapper) > 0){
            throw new ToolsBusinessWarn("文件名称重复");
        }

        // 将文件上传到minio
        InputStream inputStream = new FileInputStream(dto.getLocalFilePath());
        Attachment attachment = attachmentService.uploadAttachment(
                inputStream,
                dto.getOriginalFilename(),
                dto.getSize(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                39);
        inputStream.close();

        // 调用matlab函数读取衬套excel文件
        // 下摆臂后衬套
        this.readBushingFile(dto.getLocalFilePath(),1,attachment.getId().intValue(),0);
        // 下摆臂前衬套
        this.readBushingFile(dto.getLocalFilePath(),2,attachment.getId().intValue(),1);
        // 上摆臂衬套
        this.readBushingFile(dto.getLocalFilePath(),3,attachment.getId().intValue(),2);

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setBushingUploadStat(2);
        carBasicParam.setId(dto.getCarId());
        carBasicParamService.updateById(carBasicParam);

    }

    @SneakyThrows
    private void readBushingFile(String localFilePath,int sheetNum,int attachId,int chartType){
        LocalDateTime localDateTime = LocalDateTime.now();
        RideMatlabFunction rideMatlabFunction = new RideMatlabFunction();
        Object[] result = rideMatlabFunction.readBushingFile(7,localFilePath,sheetNum);

        MWCellArray mwCellArray = (MWCellArray) result[0];
        double[][] resultList = new double[mwCellArray.numberOfElements()][];
        for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
            resultList[i] = (double[]) mwCellArray.getCell(new int[]{1, i + 1}).toArray()[0];
        }
        MWCellArray mwCellArray2 = (MWCellArray) result[1];
        List<String> legendList = new ArrayList<>();
        for (int i = 0; i < mwCellArray2.numberOfElements(); i++) {
            String legend = mwCellArray2.getCell(new int[]{1, i + 1}).toString();
            legendList.add(legend.replace("\n"," "));
        }
        MWCellArray mwCellArray3 = (MWCellArray) result[2];
        List<String> xLabelList = new ArrayList<>();
        for (int i = 0; i < mwCellArray3.numberOfElements(); i++) {
            String legend = mwCellArray3.getCell(new int[]{1, i + 1}).toString();
            xLabelList.add(legend.replace("\n"," "));
        }
        MWCellArray mwCellArray4 = (MWCellArray) result[3];
        List<String> yLabelList = new ArrayList<>();
        for (int i = 0; i < mwCellArray4.numberOfElements(); i++) {
            String legend = mwCellArray4.getCell(new int[]{1, i + 1}).toString();
            yLabelList.add(legend.replace("\n"," "));
        }
        MWCellArray mwCellArray5 = (MWCellArray) result[4];
        double[][] yFittingData = new double[mwCellArray.numberOfElements()][];
        for (int i = 0; i < mwCellArray5.numberOfElements(); i++) {
            yFittingData[i] = (double[]) mwCellArray5.getCell(new int[]{1, i + 1}).toArray()[0];
        }
        MWCellArray mwCellArray6 = (MWCellArray) result[5];
        List<String> coefficient = new ArrayList<>();
        for (int i = 0; i < mwCellArray6.numberOfElements(); i++) {
            String legend = mwCellArray6.getCell(new int[]{1, i + 1}).toString();
            coefficient.add(legend.replace("\n"," "));
        }

        MWCellArray mwCellArray7 = (MWCellArray) result[6];
        double[][] xFittingData = new double[mwCellArray.numberOfElements()][];
        for (int i = 0; i < mwCellArray7.numberOfElements(); i++) {
            xFittingData[i] = (double[]) mwCellArray7.getCell(new int[]{1, i + 1}).toArray()[0];
        }
        int index = 0;
        List<BushingChartData> bushingChartDataList = new ArrayList<>();
        for (int i = 0; i < legendList.size(); i++) {
            ChartDataModel chartDataModel = new ChartDataModel();
            chartDataModel.setLegend(legendList.get(i));
            chartDataModel.setXLabel(xLabelList.get(i));
            chartDataModel.setYLabel(yLabelList.get(i));
            double[][] data = new double[4][];
            data[0] = resultList[index];
            data[1] = resultList[index+1];
            data[2] = xFittingData[index];
            data[3] = yFittingData[index];
            chartDataModel.setData(data);
            index += 2;
            chartDataModel.setText(coefficient.get(i));

            BushingChartData bushingChartData = new BushingChartData();
            bushingChartData.setChartType(chartType);
            bushingChartData.setChartData(FileUtil.objectToByte(chartDataModel));
            bushingChartData.setCreateTime(localDateTime);
            bushingChartData.setUpdateTime(localDateTime);
            bushingChartData.setAttachId(attachId);
            bushingChartData.setChartName(legendList.get(i));
            bushingChartDataList.add(bushingChartData);
        }
        this.saveBatch(bushingChartDataList);
    }

    /**
     * 批量删除衬套试验数据文件
     */
    @Override
    public void deleteFileList(DeleteBushingDataFileDTO dto) {
        // 删除数据库绘图数据
        LambdaQueryWrapper<BushingChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(BushingChartData::getAttachId,dto.getAttachIdList());
        this.remove(queryWrapper);

        // 删除附件
        attachmentService.removeByIds(dto.getAttachIdList());
    }

    /**
     * 查看衬套试验数据详情
     */
    @Override
    @SneakyThrows
    public ChartDataModel queryBushingData(QueryBushingDataDTO dto) {
        LambdaQueryWrapper<BushingChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(BushingChartData::getAttachId,dto.getAttachId())
                .eq(BushingChartData::getChartType,dto.getTestType())
                .eq(BushingChartData::getChartName,dto.getChartName());
        BushingChartData bushingChartData = this.getOne(queryWrapper);
        if(ObjectUtil.isEmpty(bushingChartData)){
            return null;
        }
        ChartDataModel chartDataModel = (ChartDataModel)FileUtil.byteToObject(bushingChartData.getChartData());

        // 拟合阶数不等于默认值3，重新调用matlab函数计算拟合函数数据
        if(dto.getOrder() != 1 || dto.getStartX() != -5 || dto.getEndX() != 5){
            log.info("调用matlab函数重新计算拟合数据");
            try{
                RideMatlabFunction rideMatlabFunction = new RideMatlabFunction();
                double[][] data = chartDataModel.getData();
                Object[] result = rideMatlabFunction.getFittingData3(
                        3,
                        data[0],
                        data[1],
                        dto.getOrder(),
                        dto.getStartX(),
                        dto.getEndX()
                );
                MWNumericArray mwNumericArray = (MWNumericArray)result[0];
                double[][] fittingY = (double[][])mwNumericArray.toDoubleArray();
                data[3] = fittingY[0];
                chartDataModel.setData(data);

                String text = result[1].toString();
                chartDataModel.setText(text);

                MWNumericArray mwNumericArray2 = (MWNumericArray)result[2];
                double[][] fittingX = (double[][])mwNumericArray2.toDoubleArray();
                data[2] = fittingX[0];
            }catch (Exception e){
                e.printStackTrace();
            }
            log.info("调用matlab函数重新计算拟合数据成功");
        }
        return chartDataModel;
    }

    /**
     * 对比衬套试验数据结果
     */
    @Override
    @SneakyThrows
    public CompareBushingDataVO compareBushingData(CompareBushingDataDTO dto) {
        LambdaQueryWrapper<BushingChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(BushingChartData::getAttachId,dto.getMap().keySet())
                .eq(BushingChartData::getChartType,dto.getTestType())
                .eq(BushingChartData::getChartName,dto.getChartName());
        List<BushingChartData> bushingChartDataList = this.list(queryWrapper);
        Map<Integer,List<BushingChartData>> map = bushingChartDataList.stream()
                .collect(Collectors.groupingBy(BushingChartData::getAttachId));
        Map<Integer,Map<Double,Double>> dataMap = new HashMap<>();
        for (Integer key : map.keySet()){
            List<BushingChartData> currentList = map.get(key);
            Map<Double,Double> currentMap = new HashMap<>();
            for (BushingChartData bushingChartData : currentList) {
                ChartDataModel chartDataModel = (ChartDataModel)FileUtil.byteToObject(bushingChartData.getChartData());
                double[] xData = chartDataModel.getXData();
                double[] yData = chartDataModel.getYData();
                for (int i = 0; i < xData.length && i < yData.length; i++) {
                    currentMap.put(xData[i],yData[i]);
                }
                dataMap.put(dto.getMap().get(key),currentMap);
            }
        }
        CompareBushingDataVO compareBushingDataVO = new CompareBushingDataVO();
        compareBushingDataVO.setDataMap(dataMap);
        return compareBushingDataVO;
    }
}
