package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.singularity.dao.AttachmentMapper;
import com.singularity.dao.CarKeyIndexMapper;
import com.singularity.dao.MeasureInfoMapper;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.CarKeyIndex;
import com.singularity.pojo.param.kc.CarKeyIndexCopyParam;
import com.singularity.pojo.param.kc.CarKeyIndexParam;
import com.singularity.pojo.param.kc.ExportKeyIndexParam;
import com.singularity.pojo.param.kc.SaveExcelParam;
import com.singularity.service.AttachmentService;
import com.singularity.service.CarKeyIndexService;
import com.singularity.listener.CarKeyIndexListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.*;

/**
 * @author luo
 * @description CarKeyIndex存储相关接口实现
 */
@Service
@Slf4j
public class CarKeyIndexServiceImpl implements CarKeyIndexService {
    @Resource
    private CarKeyIndexMapper carKeyIndexMapper;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private AttachmentMapper attachmentMapper;

    @Resource
    private MeasureInfoMapper measureInfoMapper;


    @Resource
    private MinioServiceImpl minioService;

    @Override
    @Transactional
    public void saveExcel(SaveExcelParam saveExcelParam) throws FileNotFoundException {
        Attachment attachment = attachmentMapper.selectById(saveExcelParam.getId());
        if (attachment == null)
            throw new RuntimeException("文件不存在");
        if (!Objects.equals(attachment.getType(), "xlsx"))
            throw new RuntimeException("文件格式错误");

        LambdaQueryWrapper<CarKeyIndex> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(CarKeyIndex::getId, saveExcelParam.getId());
        carKeyIndexMapper.delete(wrapper);


        InputStream file = minioService.getFile(attachment.getObjName());

//        InputStream in=new FileInputStream("E:\\programFile\\car\\123.xlsx");

        EasyExcel.read(file, CarKeyIndexParam.class, new CarKeyIndexListener(saveExcelParam.getId(), this)).headRowNumber(332).sheet().doRead();

    }


    @Override
    public int saveData(List<CarKeyIndex> cachedDataList) {
        return carKeyIndexMapper.insertList(cachedDataList);
    }

    @Override
    public InputStream exportKeyIndex(ExportKeyIndexParam exportKeyIndexParam) throws FileNotFoundException {
        String fileName = CarKeyIndexServiceImpl.class.getResource("/").getPath() + "Write" + ".xlsx";
//        System.out.println(fileName);
//        System.out.println(exportKeyIndexParam);
        Set<String> includeColumnFieldNames = new HashSet<>();

        List<String> name = exportKeyIndexParam.getName();


        Field[] declaredFields = CarKeyIndex.class.getDeclaredFields();

        for (Field field : declaredFields) {
            field.setAccessible(true);
            String fieldsName = field.getName();

            for (String s : name) {
                if (fieldsName.toLowerCase(Locale.ROOT).contains(s.toLowerCase(Locale.ROOT))) {
                    includeColumnFieldNames.add(fieldsName);
                }
            }

        }


        try (ExcelWriter writer = EasyExcel.write(fileName, CarKeyIndexCopyParam.class).includeColumnFieldNames(includeColumnFieldNames).build()) {

            WriteSheet writeSheet = EasyExcel.writerSheet("数据").build();
            LambdaQueryWrapper<CarKeyIndex> wrapper = new LambdaQueryWrapper<>();
            Page<CarKeyIndex> page = new Page<>(1, 200);
            wrapper.eq(CarKeyIndex::getId, exportKeyIndexParam.getId());
            Page<CarKeyIndex> pages = null;
            List<CarKeyIndex> records = null;
            List<CarKeyIndexCopyParam> carKeyIndexCopyParams = null;


            pages = carKeyIndexMapper.selectPage(page, wrapper);
            records = pages.getRecords();
            carKeyIndexCopyParams = BeanUtil.copyToList(records, CarKeyIndexCopyParam.class);
            writer.write(carKeyIndexCopyParams, writeSheet);


            for (; page.hasNext(); page.setCurrent(page.getCurrent() + 1)) {
                pages = carKeyIndexMapper.selectPage(page, wrapper);
                records = pages.getRecords();
                carKeyIndexCopyParams = BeanUtil.copyToList(records, CarKeyIndexCopyParam.class);
                writer.write(carKeyIndexCopyParams, writeSheet);
            }


            writer.finish();
        }



        return new FileInputStream(fileName);
    }


}
