package com.singularity.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.common.entity.PartChartData;
import com.singularity.dao.DamperChartDataMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.listener.DamperChartDataListener;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.damper.*;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.vo.damper.CompareDamperDataFileDetailVO;
import com.singularity.pojo.vo.damper.DamperCompareDataVO;
import com.singularity.pojo.vo.damper.QueryDamperCompareChartDataVO;
import com.singularity.pojo.vo.damper.QueryDamperDataFileDetailVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.CarBasicParamService;
import com.singularity.service.DamperChartDataService;
import com.singularity.service.DamperDataService;
import com.singularity.util.FileUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class DamperChartDataServiceImpl extends ServiceImpl<DamperChartDataMapper, PartChartData> implements DamperChartDataService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource
    private DamperDataService damperDataService;

    @Override
    @Transactional
    public void uploadDamperDataFile(UploadDamperDataFileDTO dto) {
        // 文件名验重
        attachmentService.checkRepeatFileName(
                dto.getMultipartFile().getOriginalFilename(),
                dto.getCarId(),
                FileBusinessConstant.DAMPER_DATA_FILE,
                AttachmentConstant.OWNER_TYPE_CAR
        );

        // 将附件信息保存至数据库与minio
        Attachment attachment = attachmentService.uploadAttachment(
                dto.getMultipartFile(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                FileBusinessConstant.DAMPER_DATA_FILE,
                dto.getOperId()
        );

        // 读取前减振器数据
        DamperChartDataListener frontListener = new DamperChartDataListener(
                this, attachment.getId(), dto.getCarId(),0,damperDataService
        );
        try (InputStream inputStream = dto.getMultipartFile().getInputStream()) {
            EasyExcel.read(inputStream, frontListener)
                    .sheet("前减振器")
                    .headRowNumber(1)
                    .doRead();
        } catch (Exception e) {
            e.printStackTrace();
            throw new ToolsBusinessWarn("文件格式错误！");
        }

        // 读取后减振器数据
        DamperChartDataListener rearListener = new DamperChartDataListener(
                this, attachment.getId(), dto.getCarId(),1,damperDataService
        );
        try (InputStream inputStream = dto.getMultipartFile().getInputStream()) {
            EasyExcel.read(inputStream, rearListener)
                    .sheet("后减振器")
                    .headRowNumber(1)
                    .doRead();
        } catch (Exception e) {
            throw new ToolsBusinessWarn("文件上传失败");
        }

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setDamperUploadStat(2);
        carBasicParam.setId(dto.getCarId());
        carBasicParamService.updateById(carBasicParam);
    }

    @Transactional
    @Override
    public void deleteDamperReport(DeleteDamperAttachmentDTO dto) {
        attachmentService.removeAttachmentById(
                dto.getAttachIdList()
                        .stream().map(Integer::longValue)
                        .collect(Collectors.toList()));
        LambdaQueryWrapper<PartChartData> qw = new LambdaQueryWrapper<>();
        qw.in(dto.getAttachIdList().size() > 0, PartChartData::getAttachId, dto.getAttachIdList());
        this.remove(qw);
    }

    @Override
    public QueryDamperDataFileDetailVO queryDamperDataFileDetail(QueryDamperDataFileDetailDTO dto) {
        // 查询减振器试验数据文件
        QueryDamperDataFileDetailVO result = new QueryDamperDataFileDetailVO();
        LambdaQueryWrapper<DamperData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DamperData::getAttachId,dto.getAttachId());
        List<DamperData> damperDataList = damperDataService.list(queryWrapper);
        Map<Integer,List<DamperData>> map = damperDataList.stream().collect(Collectors.groupingBy(DamperData::getDamperType));
        if(map.containsKey(0)){
            result.setFrontDamperDataList(map.get(0));
        }
        if(map.containsKey(1)){
            result.setRearDamperDataList(map.get(1));
        }
        return result;
    }

    @Override
    @SneakyThrows
    public List<ChartDataModel> queryDamperChartDataList(QueryDamperChartDataListDTO dto) {
        LambdaQueryWrapper<PartChartData> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.eq(PartChartData::getAttachId,dto.getAttachId());
        List<PartChartData> partChartDataList = this.list(queryWrapper2);
        List<ChartDataModel> chartDataModelList = new ArrayList<>();
        for (PartChartData partChartData : partChartDataList) {
            ChartDataModel chartDataModel = (ChartDataModel) FileUtil.byteToObject(partChartData.getChartData());
            chartDataModelList.add(chartDataModel);
        }
        return chartDataModelList;
    }

    @Override
    public CompareDamperDataFileDetailVO compareDamperDataFileDetail(CompareDamperDataFileDetailDTO dto) {
        // 查询减振器试验数据文件
        CompareDamperDataFileDetailVO result = new CompareDamperDataFileDetailVO();
        LambdaQueryWrapper<DamperData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(DamperData::getAttachId,dto.getMap().values());
        List<DamperData> damperDataList = damperDataService.list(queryWrapper);
        LinkedHashMap<Long,List<DamperData>> map = damperDataList.stream().collect(Collectors.groupingBy(DamperData::getCarId, LinkedHashMap::new,Collectors.toList()));
        Map<Integer,DamperCompareDataVO> frontMap = new HashMap<>();
        Map<Integer,DamperCompareDataVO> rearMap = new HashMap<>();
        for(Map.Entry<Long,List<DamperData>> entry : map.entrySet()){
            List<DamperData> damperDataList2 =  entry.getValue();
            int frontCount = 1;
            int rearCount = 1;
            for (int i = 0; i < damperDataList2.size(); i++) {
                DamperData damperData = damperDataList2.get(i);
                DamperCompareDataVO damperCompareDataVO;

                if(damperData.getDamperType() == 0 && frontMap.containsKey(frontCount)){
                    damperCompareDataVO = frontMap.get(frontCount);
                }else if(damperData.getDamperType() == 1 && rearMap.containsKey(rearCount)){
                    damperCompareDataVO = rearMap.get(rearCount);
                }else{
                    damperCompareDataVO = new DamperCompareDataVO();
                    damperCompareDataVO.setDamperType(damperData.getDamperType());
                }
                StringBuilder frequency = StringUtils.isEmpty(damperCompareDataVO.getFrequency()) ? new StringBuilder(damperData.getFrequency().toString()) : new StringBuilder(damperCompareDataVO.getFrequency() + "/" + damperData.getFrequency());
                damperCompareDataVO.setFrequency(frequency.toString());
                StringBuilder amplitude = StringUtils.isEmpty(damperCompareDataVO.getAmplitude()) ? new StringBuilder(damperData.getAmplitude().toString()) : new StringBuilder( damperCompareDataVO.getAmplitude()+ "/" + damperData.getAmplitude());
                damperCompareDataVO.setAmplitude(amplitude.toString());
                StringBuilder reboundVel = StringUtils.isEmpty(damperCompareDataVO.getReboundVel()) ? new StringBuilder(damperData.getReboundVel().toString()) : new StringBuilder(damperCompareDataVO.getReboundVel() + "/" + damperData.getReboundVel());
                damperCompareDataVO.setReboundVel(reboundVel.toString());
                StringBuilder rebound = StringUtils.isEmpty(damperCompareDataVO.getRebound()) ? new StringBuilder(damperData.getRebound().toString()) : new StringBuilder(damperCompareDataVO.getRebound() + "/" + damperData.getRebound());
                damperCompareDataVO.setRebound(rebound.toString());
                StringBuilder compressionVel = StringUtils.isEmpty(damperCompareDataVO.getCompressionVel()) ? new StringBuilder(damperData.getCompressionVel().toString()) : new StringBuilder(damperCompareDataVO.getCompressionVel() + "/" + damperData.getCompressionVel());
                damperCompareDataVO.setCompressionVel(compressionVel.toString());
                StringBuilder compression = StringUtils.isEmpty(damperCompareDataVO.getCompression()) ? new StringBuilder(damperData.getCompression().toString()) : new StringBuilder(damperCompareDataVO.getCompression() + "/" + damperData.getCompression());
                damperCompareDataVO.setCompression(compression.toString());
                if(damperData.getDamperType() == 0)
                    frontMap.put(frontCount++,damperCompareDataVO);
                if(damperData.getDamperType() == 1)
                    rearMap.put(rearCount++,damperCompareDataVO);
            }
        }
        result.setFrontDamperDataList(new ArrayList<>(frontMap.values()));
        result.setRearDamperDataList(new ArrayList<>(rearMap.values()));
        return result;
    }

    @Override
    @SneakyThrows
    public List<QueryDamperCompareChartDataVO> queryDamperCompareChartData(QueryDamperCompareChartDataDTO dto) {
        Map<Long,String> transposedMap = new HashMap<>();
        for (Map.Entry<String, Long> entry : dto.getMap().entrySet()) {
            transposedMap.put(entry.getValue(), entry.getKey());
        }
        LambdaQueryWrapper<PartChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(PartChartData::getAttachId,dto.getMap().values());
        List<PartChartData> partChartDataList = this.list(queryWrapper);
        Map<Long,List<PartChartData>> map = partChartDataList.stream().collect(Collectors.groupingBy(PartChartData::getAttachId));
        Map<String,QueryDamperCompareChartDataVO> resultMap = new HashMap<>();
        int index = 0; // 数组起始位置
        for(Map.Entry<Long,List<PartChartData>> listEntry : map.entrySet()){
            List<PartChartData> partChartDataList2 = listEntry.getValue();
            for (PartChartData partChartData : partChartDataList2) {
                ChartDataModel chartDataModel = (ChartDataModel) FileUtil.byteToObject(partChartData.getChartData());
                QueryDamperCompareChartDataVO queryDamperCompareChartDataVO;
                List<String> legendList;
                double[][] data;
                if(resultMap.containsKey(chartDataModel.getTitle())){
                    queryDamperCompareChartDataVO = resultMap.get(chartDataModel.getTitle());
                    legendList = queryDamperCompareChartDataVO.getLegendList();
                    data = queryDamperCompareChartDataVO.getData();
                }else{
                    queryDamperCompareChartDataVO = new QueryDamperCompareChartDataVO();
                    legendList = new ArrayList<>();
                    data =  new double[chartDataModel.getData().length * map.size()][];
                    queryDamperCompareChartDataVO.setChartTitle(chartDataModel.getTitle());
                }
                for (String legend : chartDataModel.getLegendList()) {
                    legendList.add(transposedMap.get(listEntry.getKey()) + legend);
                }
                System.arraycopy(chartDataModel.getData(),0,data,index * chartDataModel.getData().length,chartDataModel.getData().length);
                queryDamperCompareChartDataVO.setLegendList(legendList);
                queryDamperCompareChartDataVO.setData(data);
                resultMap.put(chartDataModel.getTitle(),queryDamperCompareChartDataVO);
            }
            index++;
        }
        return new ArrayList<>(resultMap.values());
    }
}
