package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.car.kc.DrawKCChart;
import com.mathworks.toolbox.javabuilder.MWCellArray;
import com.singularity.common.entity.ChartData;
import com.singularity.pojo.dto.kc.*;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.param.kc.*;
import com.singularity.pojo.vo.kc.*;
import com.singularity.service.*;
import com.singularity.pojo.bo.kc.CompareKCEvaluationInfoBO;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.dao.*;
import com.singularity.dao.CarDataMapper;
import com.singularity.dao.ExperimentInfoMapper;
import com.singularity.pojo.dto.experiment.ExperimentFileDTO;
import com.singularity.pojo.dto.file.DeleteNFileDTO;
import com.singularity.pojo.enumeration.CarInfoEnum;
import com.singularity.handler.kc.ExcelFillCellLineMergeHandler;
import com.singularity.pojo.vo.car.CarBasicParamVO;
import com.singularity.pojo.vo.car.CarBodyParamVO;
import com.singularity.pojo.vo.car.CarChassisStructureVO;
import com.singularity.pojo.vo.car.SelectCarEnergyParamVO;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.util.page.PageUtil;
import com.singularity.util.page.ResponseDTOWithPage;
import com.singularity.util.FileUtil;
import com.singularity.listener.KCReadFileListener;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.nio.charset.Charset;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

@Service
@Slf4j
public class KCServiceImpl implements KCService {

    @Resource
    private AttachmentMapper attachmentMapper;

    @Resource
    private CarDataMapper carDataMapper;

    @Resource
    private ExperimentInfoMapper experimentInfoMapper;

    @Resource
    private CarKeyIndexMapper carKeyIndexMapper;

    @Value("${file.path}")
    private String filePath;

    @Resource
    private MeasureInfoMapper measureInfoMapper;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private KcEvaluationService kcEvaluationService;

    @Resource
    private KcEvaluationMapper kcEvaluationMapper;

    @Resource
    private ChartDataService chartDataService;

    @Resource
    private MeasureInfoService measureInfoService;

    @Resource
    private MinioService minioService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource(name = "threadPool")
    private ThreadPoolTaskExecutor threadPool;

    @Resource
    private TaskQueueService taskQueueService;

    @Resource
    private ConditionInfoService conditionInfoService;

    @Resource
    private CarEnergyParamMapper carEnergyParamMapper;


    /**
     * KC读取数据文件
     */
    @Override
    @SneakyThrows
    public List<ReadKCFileVO> readKCFile(ReadKCFileDTO dto){
        // 1、如果设置了拟合重新调用matlab函数绘图
        if(StringUtils.isNotEmpty(dto.getXMaxPoly()) || StringUtils.isNotEmpty(dto.getXMinPoly())){
            return this.repaintChart(dto);
        }

        // 2、没有设置拟合从查询数据库查询绘图数据
        LambdaQueryWrapper<ChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ChartData::getAttachId,dto.getAttachId())
                .eq(ChartData::getMeasureCode,dto.getDrawType());
        List<ChartData> chartDataList = chartDataService.list(queryWrapper);
        if(CollectionUtil.isEmpty(chartDataList)) throw new ToolsBusinessWarn("绘图数据为空");
        ChartData chartData = chartDataList.get(0);
        Object object = FileUtil.byteToObject(chartData.getChartData());
        List<ReadKCFileVO> readKCFileVOList = new ArrayList<>();
        if (object instanceof ArrayList<?>) {
            List<?> obj = (List<?>) object;
            for (Object o : obj) {
                readKCFileVOList.add((ReadKCFileVO)o);
            }
        }
        return readKCFileVOList;
    }

    @SneakyThrows
    private List<ReadKCFileVO> repaintChart(ReadKCFileDTO dto){
        // 1、根据objName获取io流
        InputStream inputStream = minioService.getFile(dto.getObjName());
        String uuid = UUID.randomUUID().toString().replace("-", "");
        String fileNameAndPath = filePath + "/" + uuid + ".txt";
        try {
            FileUtil.downloadToLocal(inputStream, fileNameAndPath);
        } catch (IOException e) {
            e.printStackTrace();
        }

        // 2、根据测点id查询测点code，根据测点code反射调用matlab函数
        MeasureInfo measureInfo = measureInfoService.getById(dto.getMeasureId());
        DrawKCChart drawKCChart = new DrawKCChart();
        Method method = drawKCChart.getClass().getMethod(measureInfo.getMeasureCode(), int.class, Object[].class);
        Object[] object = {
                fileNameAndPath,Integer.parseInt(dto.getXMinPoly()),Integer.parseInt(dto.getXMaxPoly())
        };
        Object[] result = (Object[]) method.invoke(drawKCChart, 3, object);
        MWCellArray mwCellArray = (MWCellArray) result[0];
        double[][] resultList = new double[mwCellArray.numberOfElements()][];
        for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
            resultList[i] = (double[]) mwCellArray.getCell(new int[]{1, i + 1}).toArray()[0];
        }
        MWCellArray mwCellArray2 = (MWCellArray) result[1];
        double[][] resultList2 = new double[mwCellArray2.numberOfElements()][];
        for (int i = 0; i < mwCellArray2.numberOfElements(); i++) {
            resultList2[i] = (double[]) mwCellArray2.getCell(new int[]{1, i + 1}).toArray()[0];
        }
        MWCellArray mwCellArray3 = (MWCellArray) result[2];
        List<String> legendList = new ArrayList<>();
        for (int i = 0; i < mwCellArray3.numberOfElements(); i++) {
            String legend = mwCellArray3.getCell(new int[]{1, i + 1}).toString();
            legendList.add(legend.replace("\n"," "));
        }

        // 3、封装结果集合
        List<ReadKCFileVO> readKCFileVOList = new ArrayList<>();
        ReadKCFileVO readKCFileVO = new ReadKCFileVO();
        readKCFileVO.setData(resultList);
        readKCFileVO.setLegendList(legendList);
        readKCFileVO.setChartTitle("Front " + measureInfo.getMeasureName());
        ReadKCFileVO readKCFileVO2 = new ReadKCFileVO();
        readKCFileVO2.setData(resultList2);
        readKCFileVO2.setLegendList(legendList);
        readKCFileVO2.setChartTitle("Rear " + measureInfo.getMeasureName());
        if(readKCFileVO.getData().length != 0)readKCFileVOList.add(readKCFileVO);
        if(readKCFileVO2.getData().length != 0)readKCFileVOList.add(readKCFileVO2);
        return readKCFileVOList;
    }

    /**
     * 批量读取K&C数据文件
     */
    @Override
    @SneakyThrows
    public List<ReadKCFileListVO> readKCFileList(ReadKCFileListDTO dto){
        // 1、根据附件id与测点code查询K&C绘图对象数据
        LambdaQueryWrapper<ChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(ChartData::getChartData)
                .in(ChartData::getAttachId,dto.getAttachIdList())
                .eq(ChartData::getMeasureCode,dto.getDrawType());
        List<ChartData> chartDataList = chartDataService.list(queryWrapper);

        // 2、遍历每个附件的绘图数据
        ReadKCFileListVO readKCFileListVO1 = new ReadKCFileListVO(); // front
        ReadKCFileListVO readKCFileListVO2 = new ReadKCFileListVO(); // rear
        int i = 0;
        for (ChartData chartData : chartDataList) {
            try {
                Object object = FileUtil.byteToObject(chartData.getChartData());
                List<ReadKCFileVO> readKCFileVOList = new ArrayList<>();
                if (object instanceof ArrayList<?>) {
                    List<?> obj = (List<?>) object;
                    for (Object o : obj) {
                        readKCFileVOList.add((ReadKCFileVO)o);
                    }
                }
                ReadKCFileVO front = readKCFileVOList.get(0);
                ReadKCFileVO rear = readKCFileVOList.get(1);

                // data
                int oldFrontLength = readKCFileListVO1.getData() != null ? readKCFileListVO1.getData().length : 0;
                double[][] frontData = new double[oldFrontLength + front.getData().length][];
                if (readKCFileListVO1.getData() != null){
                    System.arraycopy(readKCFileListVO1.getData(),0,frontData,0,readKCFileListVO1.getData().length );
                    System.arraycopy(front.getData(),0,frontData,readKCFileListVO1.getData().length,front.getData().length);
                }else{
                    frontData = front.getData();
                }
                int oldRearLength = readKCFileListVO2.getData() != null ? readKCFileListVO2.getData().length : 0;
                double[][] rearData = new double[oldRearLength + rear.getData().length][];
                if(readKCFileListVO2.getData() != null){
                    System.arraycopy(readKCFileListVO2.getData(),0,rearData,0,readKCFileListVO2.getData().length);
                    System.arraycopy(rear.getData(),0,rearData,readKCFileListVO2.getData().length,rear.getData().length);
                }else{
                    rearData = rear.getData();
                }

                // legend
                List<String> carNameAndLegend1  = new ArrayList<>();
                for (String legend : front.getLegendList()) {
                    legend = dto.getCarModelList().get(i) + " " + legend;
                    carNameAndLegend1.add(legend);
                }
                List<String> frontLegendList = readKCFileListVO1.getLegendList();
                frontLegendList.addAll(carNameAndLegend1);
                List<String> carNameAndLegend2  = new ArrayList<>();
                for (String legend : rear.getLegendList()) {
                    legend = dto.getCarModelList().get(i) + " " + legend;
                    carNameAndLegend2.add(legend);
                }
                List<String> rearLegendList = readKCFileListVO2.getLegendList();
                rearLegendList.addAll(carNameAndLegend2);

                // vo
                readKCFileListVO1.setData(frontData);
                readKCFileListVO1.setChartTitle(front.getChartTitle());
                readKCFileListVO1.setLegendList(frontLegendList);
                readKCFileListVO2.setData(rearData);
                readKCFileListVO2.setChartTitle(rear.getChartTitle());
                readKCFileListVO2.setLegendList(rearLegendList);
                i++;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        List<ReadKCFileListVO> resultList = new ArrayList<>();
        resultList.add(readKCFileListVO1);
        resultList.add(readKCFileListVO2);
        return resultList;
    }

    /**
     * 车辆信息主列表分页查询
     */
    @Override
    public ResponseDTOWithPage<CarDataDTO> getCarDataList(CarDataParam carDataParam) {
        PageUtil.startPage(carDataParam, true, CarDataDTO.class);
        List<CarDataDTO> carData = carDataMapper.select(carDataParam);
        return PageUtil.getListDTO(carData, carDataParam);
    }

    @Override
    public CarDataStatisticsDTO getCarDataStatistics(CarDataStatisticsParam carDataStatisticsParam) {
        List<CarBasicParam> carData = carDataMapper.selectAll(carDataStatisticsParam);
        CarDataStatisticsDTO carDataStatisticsDTO=new CarDataStatisticsDTO();
        for (CarBasicParam item:carData) {
            /*
              获取年份统计
             */
            if (carDataStatisticsDTO.getYearMap().containsKey(item.getParticularYear())) {
                carDataStatisticsDTO.getYearMap().put(item.getParticularYear(), carDataStatisticsDTO.getYearMap().get(item.getParticularYear()) + 1);
            } else if(item.getParticularYear()!=null){
                carDataStatisticsDTO.getYearMap().put(item.getParticularYear(), 1);
            }
            /*
              获取级别统计
             */
            if (carDataStatisticsDTO.getLevelMap().containsKey(item.getVehicleLevel())) {
                carDataStatisticsDTO.getLevelMap().put(item.getVehicleLevel(), carDataStatisticsDTO.getLevelMap().get(item.getVehicleLevel()) + 1);
            } else if(item.getVehicleLevel()!=null){
                carDataStatisticsDTO.getLevelMap().put(item.getVehicleLevel(), 1);
            }

            /*
              获取级别统计
             */
            String category = CarInfoEnum.CategoryEnum.getNameByCode(item.getCategory());
            if (carDataStatisticsDTO.getCategoryMap().containsKey(category)) {
                carDataStatisticsDTO.getCategoryMap().put(category, carDataStatisticsDTO.getCategoryMap().get(category) + 1);
            } else if(item.getCategory()!=null) {
                carDataStatisticsDTO.getCategoryMap().put(category, 1);
            }

            /*
              获取国别统计
             */
            String productionPlace = CarInfoEnum.ProductionPlaceEnum.getNameByCode(item.getProductionPlace());
            if (carDataStatisticsDTO.getCountryMap().containsKey(productionPlace)) {
                carDataStatisticsDTO.getCountryMap().put(productionPlace, carDataStatisticsDTO.getCountryMap().get(productionPlace) + 1);
            } else if(productionPlace!=null){
                carDataStatisticsDTO.getCountryMap().put(productionPlace, 1);
            }

            /*
              获取能源统计
             */
            String energy = CarInfoEnum.EnergyEnum.getNameByCode(item.getEnergy());
            if (carDataStatisticsDTO.getEnergyMap().containsKey(energy)) {
                carDataStatisticsDTO.getEnergyMap().put(energy, carDataStatisticsDTO.getEnergyMap().get(energy) + 1);
            } else if(item.getEnergy()!=null){
                carDataStatisticsDTO.getEnergyMap().put(energy, 1);
            }

        }


        return carDataStatisticsDTO;
    }

    @Override
    public ExperimentConditionDTO getExperimentCondition(ExperimentConditionParam experimentConditionParam) {
        return experimentInfoMapper.selectExperimentConditionDTO(experimentConditionParam);
    }

    @Override
    public int insertExperimentCondition(SetExperimentParamDTO setExperimentParamDTO){
        return experimentInfoMapper.insertItem(setExperimentParamDTO);
    }

    @Override
    public int addCarDataList(CarDataAddParam carDataAddParam) {
        return carDataMapper.insert(carDataAddParam);
    }

    @Override
    public ResponseDTOWithPage<KeyIndexDTO> getKeyIndexList(KeyIndexParam keyIndexParam) {
        //查询第一条符号要求的数据
        CarKeyIndex carKeyIndex=carKeyIndexMapper.select(keyIndexParam);


        //查询全部measureInfo表的数据，用name匹配后缀，code匹配前缀（code可以不使用）
        LambdaQueryWrapper<MeasureInfo> measureInfoLambdaQueryWrapper=new LambdaQueryWrapper<>();
        //表中缺少单位
        List<MeasureInfo> measureInfos = measureInfoMapper.selectList(measureInfoLambdaQueryWrapper);

        //将前缀与后缀放入map
        Map<String,String> map=new TreeMap<>();
        measureInfos.forEach(item-> map.put(item.getMeasureName(),item.getUnit()));



        List<KeyIndexDTO> list=new ArrayList<>();
        try{
            //反射获取car_key_index的每一个变量
            Field[] fields=carKeyIndex.getClass().getDeclaredFields();
            for (Field field:fields){
                field.setAccessible(true);
                String name= field.getName();
                String value=field.get(carKeyIndex).toString();

                String key=null;
                String unit=null;

                //将变量名与后缀匹配
                for (Map.Entry<String, String> entry:map.entrySet()){
                    if(name.toLowerCase(Locale.ROOT).contains(entry.getKey().toLowerCase(Locale.ROOT))) {
                        key=entry.getKey();
                        unit=entry.getValue();
                        break;
                    }
                }

                String finalKey = key;
                AtomicBoolean flag= new AtomicBoolean(true);
                //在已建列表中查询是否已经创建该后缀的DTO,如果有则添加数据
                list.forEach(item->{
                    if(Objects.equals(item.getName(), finalKey) && finalKey!=null){
                        flag.set(false);
                        String[] split = name.toUpperCase().split(finalKey.toUpperCase());
                        item.getType().add(split[0]);
                        item.getKeyIndex().add(Double.valueOf(value));
                    }
                });

                //如果没有则创建
                if(flag.get() && finalKey!=null){
                    List<Double> doubles=new ArrayList<>();
                    List<String> strings=new ArrayList<>();
                    doubles.add(Double.valueOf(value));
                    String[] split = name.toUpperCase().split(key.toUpperCase());
                    strings.add(split[0]);
                    list.add(KeyIndexDTO.builder().name(key).unit(unit).keyIndex(doubles).type(strings).build());
                }


            }
        }catch (Exception ignored){

        }

        //分页返回数据
        ResponseDTOWithPage<KeyIndexDTO> responseDTOWithPage=new ResponseDTOWithPage<>();
        responseDTOWithPage.setCount((long) list.size());
        List<KeyIndexDTO> c=new ArrayList<>();
        for(int i= (keyIndexParam.getPage()-1)* keyIndexParam.getPageSize();i< keyIndexParam.getPage()* keyIndexParam.getPageSize() && i<list.size();i++){
            c.add(list.get(i));

        }
        responseDTOWithPage.setList(c);
        return responseDTOWithPage;
    }

    @Override
    public List<CarBasicParamVO> selectCarBasicParam(CarMessageIdDTO carMessageIdDTO) {
        List<CarBasicParamVO> carBasicParamVOList = attachmentMapper.selectCarBasicParam(carMessageIdDTO);
        for (CarBasicParamVO carBasicParamVO : carBasicParamVOList) {
            carBasicParamVO.setEnergyStr(CarInfoEnum.FrontHangingEnum.getNameByCode(carBasicParamVO.getEnergy()));
            carBasicParamVO.setClassificationStr(CarInfoEnum.ClassificationEnum.getNameByCode(carBasicParamVO.getClassification()));
            carBasicParamVO.setLevelStr(CarInfoEnum.LevelEnum.getNameByCode(carBasicParamVO.getLevel()));
            carBasicParamVO.setProductionPlaceStr(CarInfoEnum.ProductionPlaceEnum.getNameByCode(carBasicParamVO.getProductionPlace()));
            carBasicParamVO.setCategoryStr(CarInfoEnum.CategoryEnum.getNameByCode(carBasicParamVO.getCategory()));
        }

        return carBasicParamVOList;
    }

    @Override
    public List<CarBodyParamVO> selectCarBodyParam(CarMessageIdDTO carMessageIdDTO) {
        return attachmentMapper.selectCarBodyParam(carMessageIdDTO);
    }

    @Override
    public List<CarChassisStructureVO> selectCarChassisStructure(CarMessageIdDTO carMessageIdDTO) {
        List<CarChassisStructureVO> carChassisStructureVOList  = attachmentMapper.selectCarChassisStructure(carMessageIdDTO);
        for (CarChassisStructureVO carChassisStructureVO : carChassisStructureVOList) {
            carChassisStructureVO.setFrontHangingFormStr(CarInfoEnum.FrontHangingEnum.getNameByCode(carChassisStructureVO.getFrontHangingForm()));
            carChassisStructureVO.setRearHangingFormStr(CarInfoEnum.RearHangingEnum.getNameByCode(carChassisStructureVO.getRearHangingForm()));
            carChassisStructureVO.setSteeringTypeStr(CarInfoEnum.SteeringTypeEnum.getNameByCode(carChassisStructureVO.getSteeringType()));
            carChassisStructureVO.setRearRetardationStr(CarInfoEnum.RetardationEnum.getNameByCode(carChassisStructureVO.getRearRetardation()));
            carChassisStructureVO.setFrontRetardationStr(CarInfoEnum.RetardationEnum.getNameByCode(carChassisStructureVO.getFrontRetardation()));
        }
        return carChassisStructureVOList;
    }

    @Override
    @SneakyThrows
    public void importFileByName(KCUploadCarFileDTO kcUploadCarFileDTO) {
        EasyExcel.read(kcUploadCarFileDTO.getMultipartFile().getInputStream(), CarExcelDTO.class, new KCReadFileListener(this)).doReadAll();
    }

    @Override
    @Transactional
    public void saveData(List<CarFileDTO> cachedDataList) {
        List<String> vinList = cachedDataList.stream().map(CarFileDTO::getVIN).collect(Collectors.toList());
        // 校验excel中vin是否为空与重复
        Set<String> set = new HashSet<>();
        for (String vin : vinList) {
            if(StringUtils.isEmpty(vin)){
                throw new ToolsBusinessWarn("VIN不能为空");
            }
            if (set.contains(vin)) {
                throw new ToolsBusinessWarn("VIN不能重复");
            }
            set.add(vin);
        }

        // 校验excel中的VIN是否与数据库重复
        LambdaQueryWrapper<CarBasicParam> qw = new LambdaQueryWrapper<>();
        qw.in(CarBasicParam::getVin,vinList);
        if (carBasicParamService.list(qw).size() > 0){
            throw new ToolsBusinessWarn("已存在相同VIN");
        }

        if (cachedDataList.size() > 0){
            carDataMapper.saveBasicParam(cachedDataList);
            carDataMapper.saveBodyParam(cachedDataList);
            carDataMapper.saveStructureParam(cachedDataList);
            cachedDataList.remove(0);
        }
    }

    @Override
    public void updateCarBodyParam(CarFileDTO carMessageIdDTO) {
        carMessageIdDTO.setUpdatedAt(new Date());
        carDataMapper.updateCarBodyParam(carMessageIdDTO);
    }

    @Override
    public void updateCarBasicParam(CarFileDTO carMessageIdDTO) {
        carMessageIdDTO.setUpdatedAt(new Date());
        carDataMapper.updateCarBasicParam(carMessageIdDTO);
    }

    @Override
    public void updateCarChassisStructure(CarFileDTO carMessageIdDTO) {
        carMessageIdDTO.setUpdatedAt(new Date());
        carDataMapper.updateCarChassisStructure(carMessageIdDTO);
    }

    /**
     * 获取K&C关键指标信息
     */
    @Override
    public List<QueryKCEvaluationInfoVO> queryKCEvaluationInfo(QueryKCEvaluationInfoDTO dto){
        LambdaQueryWrapper<KcEvaluation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(KcEvaluation::getAttachId,dto.getAttachmentId());
        List<KcEvaluation> kcEvaluationList = kcEvaluationService.list(queryWrapper);
        return BeanUtil.copyToList(kcEvaluationList, QueryKCEvaluationInfoVO.class);
    }

    /**
     * 对比K&C关键指标信息
     */
    @Override
    public CompareKCEvaluationInfoVO compareKCEvaluationInfo(CompareKCEvaluationInfoDTO dto){
        CompareKCEvaluationInfoVO compareKCEvaluationInfoVO  = new CompareKCEvaluationInfoVO();

        // 1、根据车辆idList查询对比信息
        List<CompareKCEvaluationInfoBO> compareKCEvaluationInfoBOList =
                kcEvaluationMapper.compareKCEvaluationInfo(dto.getMap().keySet());

        // 2、封装表头
        Map<Integer, List<CompareKCEvaluationInfoBO>> map = compareKCEvaluationInfoBOList.stream()
                .collect(Collectors.groupingBy(CompareKCEvaluationInfoBO::getAttachId));
        if(map.size() <= 0){
            return compareKCEvaluationInfoVO;
        }
        List<CompareKCEvaluationInfoBO> firstCarData = map.get(map.keySet().iterator().next());
        List<CompareKCTableHeadVO> compareKCTableHeadVOList = new ArrayList<>();
        for (CompareKCEvaluationInfoBO compareKCEvaluationInfoBO : firstCarData) {
            CompareKCTableHeadVO compareKCTableHeadVO = new CompareKCTableHeadVO();
            List<CompareKCTableHeadVO.CompareKCTableHeadSon1> compareKCTableHeadSon1List = new ArrayList<>();
            compareKCTableHeadVO.setConditionCode(compareKCEvaluationInfoBO.getConditionCode());
            CompareKCTableHeadVO.CompareKCTableHeadSon1 compareKCTableHeadSon1 = new CompareKCTableHeadVO.CompareKCTableHeadSon1();
            compareKCTableHeadSon1.setIndicatorName(compareKCEvaluationInfoBO.getIndicatorName());
            compareKCTableHeadSon1.setUnit(compareKCEvaluationInfoBO.getUnit());
            // 封装只包含一个默认属性对象的list
//            List<CompareKCTableHeadVO.CompareKCTableHeadSon2> CompareKCTableHeadSon2List = new ArrayList<>();
//            CompareKCTableHeadVO.CompareKCTableHeadSon2 compareKCTableHeadSon2 = new CompareKCTableHeadVO.CompareKCTableHeadSon2();
//            CompareKCTableHeadSon2List.add(compareKCTableHeadSon2);
//            compareKCTableHeadSon1.setCompareKCTableHeadSon2List(CompareKCTableHeadSon2List);
            compareKCTableHeadSon1List.add(compareKCTableHeadSon1);
            compareKCTableHeadVO.setCompareKCTableHeadSon1List(compareKCTableHeadSon1List);
            compareKCTableHeadVOList.add(compareKCTableHeadVO);
        }
        compareKCEvaluationInfoVO.setTableDeadList(compareKCTableHeadVOList);

        // 3、获取对比数据集合
        Map<Integer, Map<String, List<Double>>> data  = new HashMap<>();
        for (Integer attachId : dto.getMap().keySet()) {
            List<CompareKCEvaluationInfoBO> paramList = map.get(attachId);
            if (Objects.isNull(paramList) || paramList.size() < 1) {
                continue;
            }
            Map<String, List<Double>> paramMap = new HashMap<>();
            for (CompareKCEvaluationInfoBO compareKCEvaluationInfoBO : paramList) {
                List<Double> list = new ArrayList<>();
                list.add(compareKCEvaluationInfoBO.getRightFront());
                list.add(compareKCEvaluationInfoBO.getLeftFront());
                list.add(compareKCEvaluationInfoBO.getFront1());
                list.add(compareKCEvaluationInfoBO.getRightRear());
                list.add(compareKCEvaluationInfoBO.getLeftRear());
                list.add(compareKCEvaluationInfoBO.getRear2());
                paramMap.put(compareKCEvaluationInfoBO.getConditionCode() + "-" + compareKCEvaluationInfoBO.getIndicatorName(), list);
            }
            data.put(dto.getMap().get(attachId), paramMap);
        }
        compareKCEvaluationInfoVO.setData(data);

        return compareKCEvaluationInfoVO;
    }

    /**
     * 批量删除KC附件
     */
    @Override
    public void deleteKCAttachmentByIdList(DeleteNFileDTO deleteNFileDTO){
        // 1、删除数据库附件表信息与minio附件
        attachmentService.removeAttachmentById(deleteNFileDTO.getIdList());

        // 2、删除附件绘图数据
        LambdaQueryWrapper<ChartData> queryWrapper1 = Wrappers.lambdaQuery();
        queryWrapper1.in(ChartData::getAttachId,deleteNFileDTO.getIdList());
        chartDataService.remove(queryWrapper1);

        // 3、删除附件关键指标数据
        LambdaQueryWrapper<KcEvaluation> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.in(KcEvaluation::getAttachId,deleteNFileDTO.getIdList());
        kcEvaluationService.remove(queryWrapper2);
    }

    @Override
    @SneakyThrows
    public void exportKCEvaluationInfo(ExportKCEvaluationInfoDTO dto, HttpServletResponse response) {
        // 1、查询模板信息
        LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Attachment::getBusinessType,53);
        List<Attachment> attachmentList = attachmentService.list(queryWrapper);
        if(attachmentList.size() < 1)throw new ToolsBusinessWarn("请先上传K&C关键指标导出模板");
        Attachment attachment = attachmentList.get(0);
        InputStream tempInputStream = minioService.getFile(attachment.getObjName());
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String fileName = "K&C关键指标.xlsx";
        String outPath = filePath + uuid + "-" + fileName;

        //获取指标数据
        List<kcEvaluationExcelDTO> kcEvaluationExcelDTOList = kcEvaluationMapper.queryKCEvaluationInfoList(dto);
        if (kcEvaluationExcelDTOList.size() < 1) {
            throw new ToolsBusinessWarn("未查询到数据");
        }

        // 从MTS数据中计算出ABD模板数据
        List<kcEvaluationExcelDTO>  abdDataList = this.getABDModelData(kcEvaluationExcelDTOList);

        // 合并列
        int[] mergeColumnIndex = {0,1};

        // 创建 ExcelWriter 实例
        ExcelWriter writer = EasyExcel
                // 写入到
                .write(outPath)
                // 指定写入策略
                .registerWriteHandler(new ExcelFillCellLineMergeHandler(0, mergeColumnIndex))
                // 指定模板
                .withTemplate(tempInputStream)
                .build();

        WriteSheet sheet = EasyExcel.writerSheet("K&C关键指标数据").build();

        FillConfig fillConfig = FillConfig.builder()
                // 开启填充换行
                .forceNewRow(true)
                .build();

        // 执行填充操作
        writer.fill(abdDataList, fillConfig, sheet);

        // 结束
        writer.finish();

        FileUtil.downloadToResponse(outPath, fileName, response);

        // 删除本地临时文件
        FileUtil.deleteFile(outPath);
    }

    private List<kcEvaluationExcelDTO> getABDModelData(List<kcEvaluationExcelDTO> mtsDataList){
        List<kcEvaluationExcelDTO> abdDataList = new ArrayList<>();
        Map<Integer,List<kcEvaluationExcelDTO>> map = mtsDataList.stream().collect(Collectors.groupingBy(kcEvaluationExcelDTO::getAttachId));
        int serial = 1;
        for(Map.Entry<Integer,List<kcEvaluationExcelDTO>> entry : map.entrySet()){
            for (kcEvaluationExcelDTO kcEvaluationExcelDTO : entry.getValue()) {
                kcEvaluationExcelDTO.setSerial(serial);
            }
            serial++;
        }

        for (kcEvaluationExcelDTO mtsData : mtsDataList) {
            switch (mtsData.getConditionId()) {
                case 1: // VVT
                    switch (mtsData.getIndicatorCode()) {
                        case "Wheel Rate (without tire)":
                            mtsData.setIndicatorCode("Wheel Rates");
                            mtsData.setDirection("方向");
                            abdDataList.add(mtsData);
                            break;
                        case "Radial Rate":
                            mtsData.setIndicatorCode("Tyre Radial Rates");
                            abdDataList.add(mtsData);
                            break;
                        case "Ride Rate (with tire)":
                            mtsData.setIndicatorCode("Ride Rates");
                            abdDataList.add(mtsData);
                            break;
                        case "Compliance Steer Angle Change":
                            mtsData.setIndicatorCode("BumpSteer");
                            mtsData.setDirection("正值=前束内敛");
                            mtsData.setLeftFront(mtsData.getLeftFront() * -1);
                            mtsData.setLeftRear(mtsData.getLeftRear() * -1);
                            mtsData.setFront1(mtsData.getFront1() * -1);
                            mtsData.setRightFront(mtsData.getRightFront() * -1);
                            mtsData.setRightRear(mtsData.getRightFront() * -1);
                            mtsData.setRear2(mtsData.getRear2() * -1);
                            abdDataList.add(mtsData);
                            break;
                        case "Camber Angle Change":
                            mtsData.setIndicatorCode("BumpCamber");
                            mtsData.setDirection("正值=顶部向外");
                            mtsData.setLeftFront(mtsData.getLeftFront() * -1);
                            mtsData.setLeftRear(mtsData.getLeftRear() * -1);
                            mtsData.setFront1(mtsData.getFront1() * -1);
                            mtsData.setRightFront(mtsData.getRightFront() * -1);
                            mtsData.setRightRear(mtsData.getRightFront() * -1);
                            mtsData.setRear2(mtsData.getRear2() * -1);
                            abdDataList.add(mtsData);
                            break;
                        case "Spin Angle Change":
                            mtsData.setIndicatorCode("BumpSpin");
                            mtsData.setDirection("正值=顶部向前");
                            mtsData.setLeftFront(mtsData.getLeftFront() * -1);
                            mtsData.setLeftRear(mtsData.getLeftRear() * -1);
                            mtsData.setFront1(mtsData.getFront1() * -1);
                            mtsData.setRightFront(mtsData.getRightFront() * -1);
                            mtsData.setRightRear(mtsData.getRightFront() * -1);
                            mtsData.setRear2(mtsData.getRear2() * -1);
                            abdDataList.add(mtsData);
                            break;
                        case "Lateral Wheel Center Locus (Z vs Y )":
                            mtsData.setIndicatorCode("Lateral WheelCenter Displacement");
                            mtsData.setDirection("正值=变宽");
                            abdDataList.add(mtsData);
                            break;
                        case "Longitudinal Wheel Center Locus (Z vs X )":
                            mtsData.setIndicatorCode("Wheel Recession");
                            mtsData.setDirection("正值=变宽");
                            mtsData.setLeftFront(mtsData.getLeftFront() * -1000);
                            mtsData.setLeftRear(mtsData.getLeftRear() * -1000);
                            mtsData.setFront1(mtsData.getFront1() * -1000);
                            mtsData.setRightFront(mtsData.getRightFront() * -1000);
                            mtsData.setRightRear(mtsData.getRightFront() * -1000);
                            mtsData.setRear2(mtsData.getRear2() * -1000);
                            abdDataList.add(mtsData);
                            break;
                        case "Track Change":
                            mtsData.setIndicatorCode("Lateral WheelCenter Displacement");
                            mtsData.setDirection("正值=变宽");
                            mtsData.setLeftFront(mtsData.getLeftFront() * 1000);
                            mtsData.setLeftRear(mtsData.getLeftRear() * -1000);
                            mtsData.setRightFront(mtsData.getRightFront() * -1000);
                            mtsData.setRightRear(mtsData.getRightFront() * -1000);
                            abdDataList.add(mtsData);
                            break;
                        case "Roll Center":
                            mtsData.setIndicatorCode("Kinematic Roll Centre Height");
                            mtsData.setLeftFront(mtsData.getLeftFront() * -1);
                            mtsData.setLeftRear(mtsData.getLeftRear() * -1);
                            mtsData.setFront1(mtsData.getFront1() * -1);
                            mtsData.setRightFront(mtsData.getRightFront() * -1);
                            mtsData.setRightRear(mtsData.getRightFront() * -1);
                            mtsData.setRear2(mtsData.getRear2() * -1);
                            abdDataList.add(mtsData);
                            break;
                        case "Evaluated at Vert Position(mm):0":
                            mtsData.setIndicatorCode("Kinematic Anti-Dive Angle");
                            mtsData.setDirection("正值利于抗点头　");
                            abdDataList.add(mtsData);
                            break;
                        case "Hystersis:Wheel Rates(without tire)":
                            mtsData.setIndicatorCode("Hystersis:Wheel Rates");
                            mtsData.setDirection("正值=变宽");
                            abdDataList.add(mtsData);
                            break;
                    }
                    break;
                case 29: // VRT
                    switch (mtsData.getIndicatorCode()) {
                        case "Front Roll Monment":
                        mtsData.setIndicatorCode("Roll Siffness");
                        mtsData.setLeftFront(mtsData.getLeftFront() * -100);
                        mtsData.setLeftRear(mtsData.getLeftRear() * -100);
                        mtsData.setFront1(mtsData.getFront1() * -100);
                        mtsData.setRightFront(mtsData.getRightFront() * -100);
                        mtsData.setRightRear(mtsData.getRightFront() * -100);
                        mtsData.setRear2(mtsData.getRear2() * -100);
                        abdDataList.add(mtsData);

                        break;
                        case "Wheel Load(On Roll Angle)":
                            mtsData.setIndicatorCode("Wheel VerLoads V Roll Angle");
                            abdDataList.add(mtsData);
                            break;
                        case "Wheel Rates":
                            mtsData.setIndicatorCode("Spring Rate (without tire)");
                            abdDataList.add(mtsData);
                            break;
                        case "Compliance Steer Angle Change (Steer/Roll)":
                            mtsData.setIndicatorCode("Roll Steer V Roll Angle");
                            abdDataList.add(mtsData);
                            break;
                        case "Compliance Camber Angle Change (Camebr/Roll)":
                            mtsData.setIndicatorCode("Roll Camber V Roll Angle");
                            abdDataList.add(mtsData);
                            break;
                        case "Compliance Camber (through Z0)":
                            mtsData.setIndicatorCode("Roll Camber V Wheel to Body");
                            abdDataList.add(mtsData);
                            break;
                    }
                    break;
                case 33: // LAT
                    switch (mtsData.getIndicatorCode()) {
                        case "Lat Stiffness - Wheel Center":
                            mtsData.setIndicatorCode("Lateral wheelCenter Compliance");
                            abdDataList.add(mtsData);
                            break;
                        case "Compliance Steer Angle Change":
                            mtsData.setIndicatorCode("Latreal Toe Compliance");
                            abdDataList.add(mtsData);
                            break;
                        case "Compliance Camber Angle Change":
                            mtsData.setIndicatorCode("Latreal Camber Compliance");
                            abdDataList.add(mtsData);
                            break;
                        case "Roll Center (Fz/Fy)":
                            mtsData.setIndicatorCode("Roll centre Curve Fits");
                            abdDataList.add(mtsData);
                            break;
                        case "Height(mm)":
                            mtsData.setIndicatorCode("Force Roll Centre Height");
                            abdDataList.add(mtsData);
                            break;
                    }
                    break;
                case 38: //BAT
                    switch (mtsData.getIndicatorCode()) {
                        case "Long Stiffness Wheel Center":
                            mtsData.setIndicatorCode("LongWheelCenterCompliance");
                            abdDataList.add(mtsData);
                            break;
                        case "Steer Angle Change":
                            mtsData.setIndicatorCode("Longitudinal Toe Compliance");
                            abdDataList.add(mtsData);
                            break;
                        case "Longitudinal Spin Compllance":
                            mtsData.setIndicatorCode("Camber Angle Change");
                            abdDataList.add(mtsData);
                            break;
                        case "AntiDive/Squat Curve Fits":
                            mtsData.setIndicatorCode("Anti-Dive / Anti-Squat");
                            abdDataList.add(mtsData);
                            break;
                        case "Force Anti.Dive Angles":
                            mtsData.setIndicatorCode("Anti-Dive/Anti-Squat(N Fz/N Fx)");
                            abdDataList.add(mtsData);
                            break;
                    }
                    break;
                case 41: // ATA
                    switch (mtsData.getIndicatorCode()) {
                        case "Steer Angle Change":
                            mtsData.setIndicatorCode("Steer Aangle change");
                            abdDataList.add(mtsData);
                            break;
                        case "Camber Change":
                            mtsData.setIndicatorCode("Camber change");
                            abdDataList.add(mtsData);
                            break;
                    }
                    break;
                case 31: //SRT
                    switch (mtsData.getIndicatorCode()) {
                        case "SteerRatio":
                            mtsData.setIndicatorCode("On Center Steer Ratio (+/ 20°) deg SWA/ deg Steer");
                            abdDataList.add(mtsData);
                            break;
                        case "Camber Angle Change":
                            mtsData.setIndicatorCode("CamberAngle V wheelsteer");
                            abdDataList.add(mtsData);
                            break;
                        case "Kingpin Angle vs Hdwheel steer":
                            mtsData.setIndicatorCode("Kingpin Caster Ang v Hdwheel steer");
                            abdDataList.add(mtsData);
                            break;
                        case "Kingpin inclin Ang v Hdwheel steer":
                            mtsData.setIndicatorCode("Caster Angle steeringwheelangle");
                            abdDataList.add(mtsData);
                            break;
                    }
                    break;
            }
        }
        return abdDataList;
    }

    /**
     * 异步上传K&C试验数据文件
     */
    @Override
    public void asyncUploadKCConditions(ExperimentFileDTO experimentFileDTO){
        // 创建队列信息
        LocalDateTime localDateTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(experimentFileDTO.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(experimentFileDTO.getCarId());
        taskQueue.setStat(1);
        taskQueue.setOperId(experimentFileDTO.getOperId());
        taskQueueService.save(taskQueue);
        experimentFileDTO.setStartTime(localDateTime);
        experimentFileDTO.setTaskId(taskQueue.getId());

        // 异步调用任务
        threadPool.execute(() -> {
            try {
                // 核心任务
                this.uploadKCConditions(experimentFileDTO);

                // 任务执行完毕修改任务状态
                log.info("文件上传完毕");
                LocalDateTime now = LocalDateTime.now();
                Duration duration = Duration.between(experimentFileDTO.getStartTime(),now);
                LambdaUpdateWrapper<TaskQueue> updateWrapper = Wrappers.lambdaUpdate();
                updateWrapper.set(TaskQueue::getStat,2)
                        .set(TaskQueue::getConsumeTime,duration.toMillis()/1000)
                        .eq(TaskQueue::getId,experimentFileDTO.getTaskId());
                taskQueueService.update(updateWrapper);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
                // 修改任务状态
                LambdaUpdateWrapper<TaskQueue> updateWrapper = Wrappers.lambdaUpdate();
                updateWrapper.set(TaskQueue::getStat,3)
                        .set(TaskQueue::getRemark,e.getMessage())
                        .eq(TaskQueue::getId,taskQueue.getId());
                taskQueueService.update(updateWrapper);
            }finally {
                FileUtil.deleteFile(experimentFileDTO.getLocalFilePath());
            }
        });
    }
    @Transactional
    public void uploadKCConditions(ExperimentFileDTO experimentFileDTO) throws Exception{
        // 1、获取文件格式
        File file = new File(experimentFileDTO.getLocalFilePath());
        String originalFilename = file.getName();
        String[] splitFileName = originalFilename.split("\\.");
        // 文件格式
        String fileType = splitFileName[splitFileName.length - 1];

        // 调用matlab函数读取文件关键指标与绘图数据
        if ("zip".equals(fileType)) {
            // 解析zip文件
            InputStream inputStream = new FileInputStream(file);
            ZipInputStream zipInputStream = new ZipInputStream(inputStream, Charset.forName("GBK"));
            ZipEntry zipEntry = zipInputStream.getNextEntry();
            while (zipEntry != null) {
                if (!zipEntry.isDirectory()) {
                    // 截取文件后缀，校验文件是否是.tab数据文件
                    String[] filePathArray = zipEntry.getName().split("/");
                    String fileName = filePathArray[filePathArray.length - 1];
                    String[] fileNameArray = fileName.split("\\.");
                    String conditionType = fileNameArray[fileNameArray.length - 1];
                    if (!"tab".equals(conditionType)) throw new ToolsBusinessWarn("请选择.tab格式数据文件");

                    // 截取文件名，检验该文件是否是符合规范的文件
                    String conditionCode = filePathArray[filePathArray.length - 2];
                    LambdaQueryWrapper<ConditionInfo> queryWrapper = Wrappers.lambdaQuery();
                    queryWrapper.eq(ConditionInfo::getConditionCode, conditionCode);
                    ConditionInfo conditionInfo = conditionInfoService.getOne(queryWrapper);
                    // 无法匹配响应工况，跳过
                    if (ObjectUtil.isEmpty(conditionInfo)) {
                        throw new ToolsBusinessWarn("请根据规范命名文件，错误文件：" + conditionCode);
                    }


                    // 将zip中的文件下载到本地
                    InputStream zipEntryStream = new ByteArrayInputStream(IOUtils.toByteArray(zipInputStream));
                    String uuid = UUID.randomUUID().toString().replace("-", "");
                    String changeFilePath = filePath + "/" + uuid + ".tab";
                    try(FileOutputStream outputStream = new FileOutputStream(changeFilePath)){
                        byte[] buff = new byte[1024];
                        int len;
                        while((len = zipEntryStream.read(buff)) != -1) {
                            outputStream.write(buff,0,len);
                        }
                    }catch (Exception e){
                        throw new ToolsBusinessWarn("转换失败");
                    }

                    // 校验文件名称是否重复
                    LambdaQueryWrapper<Attachment> queryWrapper2 = Wrappers.lambdaQuery();
                    queryWrapper2.eq(Attachment::getName,experimentFileDTO.getOriginalFilename())
                            .eq(Attachment::getOwnerId,experimentFileDTO.getOwnerId());
                    if(attachmentService.count(queryWrapper2) > 0)throw new ToolsBusinessWarn("文件名称重复");

                    // 将本地文件上传到minio
                    Attachment attachment = attachmentService.uploadAttachment(
                            new FileInputStream(changeFilePath),
                            fileName,
                            zipEntry.getSize(),
                            experimentFileDTO.getOwnerType(),
                            experimentFileDTO.getOwnerId(),
                            experimentFileDTO.getBusinessType(),
                            experimentFileDTO.getOperId()
                    );

                    // 设置试验场景
                    ExperimentInfo experimentInfo = ExperimentInfo.builder()
                            .attachId(attachment.getId())
                            .conditionId(conditionInfo.getId())
                            .build();
                    experimentInfoMapper.insert(experimentInfo);

                    // 调用matlab函数解析关键指标数据与绘图数据
                    this.readKCFileData(new FileInputStream(changeFilePath),conditionInfo.getMethodName(),attachment.getId(),
                            experimentFileDTO.getCarId(),conditionInfo.getId());

                    // 删除本地临时File
                    FileUtil.deleteFile(changeFilePath);
                }
                zipInputStream.closeEntry();
                zipEntry = zipInputStream.getNextEntry();
            }
            zipInputStream.close();
            inputStream.close();
        }else{
            throw new ToolsBusinessWarn("文件格式错误");
        }

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setKcUploadStat(2);
        carBasicParam.setId(experimentFileDTO.getCarId());
        carBasicParamService.updateById(carBasicParam);
    }

    @Transactional
    public void uploadKCFileData(UploadKCDataFile uploadKCDataFile){
        // 校验文件格式
        MultipartFile multipartFile = uploadKCDataFile.getMultipartFile();
        String originalFilename = multipartFile.getOriginalFilename();
        assert originalFilename != null;
        String[] splitFileName = originalFilename.split("\\.");
        // 文件格式
        String fileType = splitFileName[splitFileName.length - 1];
        if("tab".equals(fileType)){ // 解析单个.tab数据文件
            // 校验文件名称是否重复
            LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.eq(Attachment::getName,originalFilename)
                    .eq(Attachment::getOwnerId,uploadKCDataFile.getCarId());
            if(attachmentService.count(queryWrapper) > 0)throw new ToolsBusinessWarn("文件名称重复");

            // 上传文件
            Attachment attachment = attachmentService.uploadAttachment(
                    multipartFile,
                    AttachmentConstant.OWNER_TYPE_CAR,
                    uploadKCDataFile.getCarId(),
                    FileBusinessConstant.KC_DATA_FILE,
                    uploadKCDataFile.getOperId()
                    );

            // 设置试验场景
            ExperimentInfo experimentInfo = ExperimentInfo.builder()
                    .attachId(attachment.getId())
                    .conditionId(uploadKCDataFile.getConditionId())
                    .build();
            experimentInfoMapper.insert(experimentInfo);

            // 调用matlab函数读取K&C绘图数据与关键指标，并保存到数据库
            ConditionInfo conditionInfo = conditionInfoService.getById(uploadKCDataFile.getConditionId());
            try{
                this.readKCFileData(multipartFile.getInputStream(),conditionInfo.getMethodName(),attachment.getId(),
                        uploadKCDataFile.getCarId(),conditionInfo.getId());
            }catch(Exception e){
                e.printStackTrace();
                throw new ToolsBusinessWarn("调用matlab函数失败");
            }

        }else{
            throw new ToolsBusinessWarn("文件格式错误");
        }

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setKcUploadStat(2);
        carBasicParam.setId(uploadKCDataFile.getCarId());
        carBasicParamService.updateById(carBasicParam);
    }

    /**
     * 查询车辆动力参数
     */
    @Override
    public List<SelectCarEnergyParamVO> selectCarEnergyParam(CarMessageIdDTO carMessageIdDTO) {
        LambdaQueryWrapper<CarEnergyParam> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(CarEnergyParam::getCarId,carMessageIdDTO.getCarId().get(0));
        List<CarEnergyParam> carEnergyParamList = carEnergyParamMapper.selectList(queryWrapper);
        return BeanUtil.copyToList(carEnergyParamList,SelectCarEnergyParamVO.class);
    }

    /**
     * 修改车辆动力参数
     */
    @Override
    public void updateCarEnergyParam(CarFileDTO carFileDTO) {
        CarEnergyParam carEnergyParam = new CarEnergyParam();
        BeanUtil.copyProperties(carFileDTO,carEnergyParam);
        LambdaQueryWrapper<CarEnergyParam> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(CarEnergyParam::getCarId,carFileDTO.getId());
        carEnergyParamMapper.update(carEnergyParam,queryWrapper);
    }


    /**
     * 调用matlab函数读取K&C绘图数据与关键指标，并保存到数据库
     */
    @SneakyThrows
    private void readKCFileData(InputStream inputStream ,String methodName,
                                Long attachId,Long carId,Integer conditionId) {
        // 1、文件格式转化，将.tab格式数据文件转换为.txt
        String uuid = UUID.randomUUID().toString().replace("-", "");
        String fileNameAndPath = filePath + "/" + uuid + ".txt";
        FileUtil.downloadToLocal(inputStream, fileNameAndPath);

        // 2、调用matlab函数生成关键指标
        log.info("开始调用matlab函数生成关键指标");
        DrawKCChart drawKCChart = new DrawKCChart();
        if (StringUtils.isNotEmpty(methodName)) {
            // 关键指标集合
            List<KcEvaluation> kcEvaluationList = new ArrayList<>();
            Method method = drawKCChart.getClass().getMethod(methodName, int.class, Object[].class);
            Object[] param = {
                    fileNameAndPath
            };
            Object[] result = (Object[]) method.invoke(drawKCChart, 1, param);
            MWCellArray mwCellArray = (MWCellArray) result[0];
            for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
                Object[] data = mwCellArray.getCell(new int[]{1, i + 1}).toArray();
                KcEvaluation kcEvaluation = new KcEvaluation();
                kcEvaluation.setIndicatorName(data[0].toString());
                kcEvaluation.setUnit(data[1].toString());
                kcEvaluation.setSlope(data[2].toString());
                kcEvaluation.setRightFront(Double.parseDouble(data[3].toString()));
                kcEvaluation.setLeftFront(Double.parseDouble(data[4].toString()));
                kcEvaluation.setFront1(Double.parseDouble(data[5].toString()));
                kcEvaluation.setRightRear(Double.parseDouble(data[6].toString()));
                kcEvaluation.setLeftRear(Double.parseDouble(data[7].toString()));
                kcEvaluation.setRear2(Double.parseDouble(data[8].toString()));
                kcEvaluation.setCarId(carId);
                kcEvaluation.setAttachId(attachId);
                kcEvaluation.setConditionId(conditionId);
                kcEvaluationList.add(kcEvaluation);
            }

            // 3、将关键指标保存到数据库
            kcEvaluationService.saveBatch(kcEvaluationList);
            log.info("关键指标生成完毕");
        }

        // 4、根据工况查询测点列表
        LambdaQueryWrapper<MeasureInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(MeasureInfo::getConditionId, conditionId)
                .eq(MeasureInfo::getStatus, 1);
        List<MeasureInfo> measureInfoList = measureInfoService.list(queryWrapper);

        // 5、调用matlab函数读取excel读取绘图数据
        for (MeasureInfo measureInfo : measureInfoList) {
            try {
                log.info("开始读取" + measureInfo.getMethodName());
                Method method2 = drawKCChart.getClass().getMethod(measureInfo.getMethodName(), int.class, Object[].class);
                Object xMinPoly = "";
                Object xMaxPoly = "";
                Object[] object = {
                        fileNameAndPath, xMinPoly, xMaxPoly
                };
                Object[] result2 = (Object[]) method2.invoke(drawKCChart, 3, object);
                MWCellArray mwCellArray2 = (MWCellArray) result2[0];
                double[][] resultList = new double[mwCellArray2.numberOfElements()][];
                for (int i = 0; i < mwCellArray2.numberOfElements(); i++) {
                    resultList[i] = (double[]) mwCellArray2.getCell(new int[]{1, i + 1}).toArray()[0];
                }
                MWCellArray mwCellArray3 = (MWCellArray) result2[1];
                double[][] resultList2 = new double[mwCellArray3.numberOfElements()][];
                for (int i = 0; i < mwCellArray3.numberOfElements(); i++) {
                    resultList2[i] = (double[]) mwCellArray3.getCell(new int[]{1, i + 1}).toArray()[0];
                }
                MWCellArray mwCellArray4 = (MWCellArray) result2[2];
                List<String> legendList = new ArrayList<>();
                for (int i = 0; i < mwCellArray4.numberOfElements(); i++) {
                    String legend = mwCellArray4.getCell(new int[]{1, i + 1}).toString();
                    legendList.add(legend.replace("\n", " "));
                }
                // 封装结果集
                List<ReadKCFileVO> readKCFileVOList = new ArrayList<>();
                ReadKCFileVO readKCFileVO = new ReadKCFileVO();
                readKCFileVO.setData(resultList);
                readKCFileVO.setLegendList(legendList);
                readKCFileVO.setChartTitle("Front " + measureInfo.getMeasureCode());
                ReadKCFileVO readKCFileVO2 = new ReadKCFileVO();
                readKCFileVO2.setData(resultList2);
                readKCFileVO2.setLegendList(legendList);
                readKCFileVO2.setChartTitle("Rear " + measureInfo.getMeasureCode());
                if (readKCFileVO.getData().length != 0) readKCFileVOList.add(readKCFileVO);
                if (readKCFileVO2.getData().length != 0) readKCFileVOList.add(readKCFileVO2);

                // 将绘图数据保存至数据库
                LocalDate localDate = LocalDate.now();
                ChartData chartData = new ChartData();
                chartData.setCreateTime(localDate);
                chartData.setUpdateTime(localDate);
                chartData.setAttachId(attachId);
                chartData.setMeasureCode(measureInfo.getMeasureCode());
                chartData.setChartData(FileUtil.objectToByte(readKCFileVOList));
                chartDataService.save(chartData);
            } catch (Exception e) {
                log.error(measureInfo.getMeasureCode() + "出错");
                e.printStackTrace();
            }
        }

        // 删除本地file
        FileUtil.deleteFile(fileNameAndPath);
        log.info("文件上传完毕");
    }
}
