package com.singularity.service.impl;

import cn.dev33.satoken.exception.SaTokenException;
import cn.dev33.satoken.secure.BCrypt;
import cn.dev33.satoken.session.SaSession;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import com.singularity.pojo.dto.permission.PermissionInfoDTO;
import com.singularity.pojo.dto.role.RoleInfoDTO;
import com.singularity.pojo.dto.user.UserInfoDTO;
import com.singularity.pojo.entity.User;
import com.singularity.pojo.enumeration.PrefixEnum;
import com.singularity.pojo.param.user.CaptchaLoginParam;
import com.singularity.pojo.param.user.PasswordLoginParam;
import com.singularity.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;

@Slf4j
@Service
public class LoginServiceImpl  implements LoginService{

    @Resource
    private UserService userService;

    @Resource
    private RoleService roleService;

    @Resource
    private PermissionService permissionService;

    @Resource(name = "threadPool")
    private Executor executor;

    @Override
    public UserInfoDTO loginByCaptcha(CaptchaLoginParam captchaLoginParam) {
        return null;
    }

    @Override
    public UserInfoDTO loginByPassword(PasswordLoginParam passwordLoginParam) {
        Long phone = passwordLoginParam.getPhone();
        String password = passwordLoginParam.getPassword();
        User user = userService.getUserByPhone(phone);
        if (Objects.isNull(user)) {
            throw new SaTokenException(400, "该手机号不存在");
        }

        if (!BCrypt.checkpw(password, user.getPassword())) {
            throw new SaTokenException(400, "密码错误");
        }
        return this.publicLogin(user);
    }

    @Override
    public void logout() {
        StpUtil.logout();
    }

    // 公用登录方法
    private UserInfoDTO publicLogin(User user) {
        Integer userId = user.getId();
        String loginId = PrefixEnum.USER.getValue() + userId;
        StpUtil.login(loginId);

        //添加角色与权限
        List<RoleInfoDTO> roles = roleService.getRoleInfo(userId);
        List<PermissionInfoDTO> permissionInfoDTOList = permissionService.getPermissionInfo(roles);
        UserInfoDTO userInfoDTO = BeanUtil.copyProperties(user, UserInfoDTO.class);
        userInfoDTO.setRoles(roles);
        userInfoDTO.setPermissions(permissionInfoDTOList);
        //异步序列化进redis
        SaSession session = StpUtil.getSession();
        CompletableFuture.runAsync(() -> session.set(SaSession.USER, user)
                .set(SaSession.ROLE_LIST, roles)
                .set(SaSession.PERMISSION_LIST, permissionInfoDTOList),executor);
        return userInfoDTO;
    }
}
