package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.singularity.dao.MeasureInfoMapper;
import com.singularity.pojo.dto.measure.*;
import com.singularity.pojo.entity.MeasureInfo;
import com.singularity.pojo.vo.subjectiveEvaluation.measure.QueryMeasureInfoListVO;
import com.singularity.pojo.vo.subjectiveEvaluation.measure.QueryMeasureListByConditionIdVO;
import com.singularity.service.MeasureInfoService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.List;

@Service
public class MeasureInfoServiceImpl extends ServiceImpl<MeasureInfoMapper, MeasureInfo> implements MeasureInfoService {

    @Resource
    private MeasureInfoMapper measureInfoMapper;

    /**
     * 测点列表分页查询
     */
    @Override
    public List<QueryMeasureInfoListVO> queryMeasureInfoList(QueryMeasureInfoListDTO dto){
        PageHelper.startPage(dto.getPageNum(),dto.getPageSize());
        return measureInfoMapper.queryMeasureInfoList(dto);
    }

    /**
     * 增加测点信息
     */
    @Override
    public void addMeasureInfo(AddMeasureInfoDTO dto){
        MeasureInfo measureInfo = new MeasureInfo();
        LocalDate date = LocalDate.now();
        BeanUtil.copyProperties(dto,measureInfo);
        measureInfo.setConditionId(dto.getConditionId());
        measureInfo.setCreateTime(date);
        measureInfo.setUpdateTime(date);
        measureInfo.setStatus(1);
        this.save(measureInfo);
    }

    /**
     * 删除测点信息
     */
    @Override
    public void deleteMeasureInfo(DeleteMeasureInfoDTO dto){
        this.removeBatchByIds(dto.getIdList());
    }

    /**
     * 修改测点信息
     */
    @Override
    public void updateMeasureInfo(UpdateMeasureInfoDTO dto){
        MeasureInfo measureInfo = new MeasureInfo();
        BeanUtil.copyProperties(dto,measureInfo);
        LocalDate date = LocalDate.now();
        measureInfo.setUpdateTime(date);
        measureInfo.setConditionId(dto.getConditionId());
        this.updateById(measureInfo);
    }

    /**
     * 根据工况id查询测点列表
     */
    @Override
    public List<QueryMeasureListByConditionIdVO> queryMeasureListByConditionId(QueryMeasureListByConditionIdDTO dto){
        LambdaQueryWrapper<MeasureInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(MeasureInfo::getId,MeasureInfo::getMeasureName,MeasureInfo::getMeasureCode).
                eq(MeasureInfo::getConditionId,dto.getConditionId()).
                eq(MeasureInfo::getStatus,1);
        List<MeasureInfo> measureInfoList = this.list(queryWrapper);
        return BeanUtil.copyToList(measureInfoList,QueryMeasureListByConditionIdVO.class);
    }
}
