package com.singularity.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.dao.MenuRoleMapper;
import com.singularity.dao.RoleMapper;
import com.singularity.pojo.dto.role.RoleInfoDTO;
import com.singularity.pojo.entity.MenuRole;
import com.singularity.pojo.entity.Role;
import com.singularity.pojo.param.role.RoleAddParam;
import com.singularity.pojo.param.role.RoleDeleteParam;
import com.singularity.pojo.param.role.RoleListSearchParam;
import com.singularity.pojo.param.role.RoleSetParam;
import com.singularity.pojo.vo.role.MenuInfo;
import com.singularity.pojo.vo.role.RoleListVO;
import com.singularity.service.MenuRoleService;
import com.singularity.service.RoleService;
import com.singularity.util.page.PageUtil;
import com.singularity.util.page.ResponseDTOWithPage;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class RoleServiceImpl implements RoleService {
    private final RoleMapper roleMapper;

    private final MenuRoleService menuRoleService;

    private final MenuRoleMapper menuRoleMapper;

    @Autowired
    public RoleServiceImpl(RoleMapper roleMapper, MenuRoleService menuRoleService,MenuRoleMapper menuRoleMapper) {
        this.roleMapper = roleMapper;
        this.menuRoleService = menuRoleService;
        this.menuRoleMapper = menuRoleMapper;
    }

    @Override
    public List<RoleInfoDTO> getRoleInfo(Integer userId) {
        return roleMapper.selectByUserId(userId);
    }

    @Override
    public ResponseDTOWithPage<RoleListVO> selectRoleList(RoleListSearchParam param) {
        PageUtil.startPage(param,true, RoleListVO.class);
        List<RoleListVO> roleListVOList = roleMapper.selectRoleList(param);

        // 查询菜单权限信息
        List<MenuInfo> menuInfoList = menuRoleMapper.queryMenuInfoList();
        // 根据角色id分组
        Map<Long, List<MenuInfo>> map = menuInfoList.stream().collect(Collectors.groupingBy(MenuInfo::getRoleId));
        // 插入菜单权限数据
        for (RoleListVO roleListVO : roleListVOList) {
            if(map.containsKey(roleListVO.getId())){
                roleListVO.setMenuList(map.get(roleListVO.getId()));
            }
        }
        return PageUtil.getListDTO(roleListVOList,param);
    }

    @Transactional
    @Override
    public void addRole(RoleAddParam roleAddParam) {
        // 添加角色基本信息
        Role role = new Role();
        BeanUtils.copyProperties(roleAddParam,role);
        roleMapper.insert(role);

        // 2、给菜单增加角色权限
        LocalDateTime localDateTime = LocalDateTime.now();
        List<MenuRole> menuRoleList = new ArrayList<>();
        roleAddParam.getMenuIdList().forEach(menuId ->{
            MenuRole menuRole = new MenuRole();
            menuRole.setRoleId(role.getId().intValue());
            menuRole.setMenuId(menuId);
            menuRole.setCreateTime(localDateTime);
            menuRole.setUpdateTime(localDateTime);
            menuRoleList.add(menuRole);
        });
        menuRoleService.saveBatch(menuRoleList);
    }

    @Transactional
    @Override
    public void setRole(RoleSetParam roleSetParam) {
        // 修改角色基本信息
        Role role = new Role();
        BeanUtils.copyProperties(roleSetParam,role);
        roleMapper.updateById(role);

        // 修改角色-菜单权限
        if(ObjectUtil.isNotEmpty(roleSetParam.getMenuIdList())){
            // 删除所有信息
            LambdaQueryWrapper<MenuRole> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.eq(MenuRole::getRoleId,roleSetParam.getId());
            menuRoleService.remove(queryWrapper);
            // 增加信息角色信息
            LocalDateTime localDateTime = LocalDateTime.now();
            List<MenuRole> menuRoleList = new ArrayList<>();
            roleSetParam.getMenuIdList().forEach(menuId ->{
                MenuRole menuRole = new MenuRole();
                menuRole.setRoleId(role.getId().intValue());
                menuRole.setMenuId(menuId);
                menuRole.setCreateTime(localDateTime);
                menuRole.setUpdateTime(localDateTime);
                menuRoleList.add(menuRole);
            });
            menuRoleService.saveBatch(menuRoleList);
        }
    }

    @Override
    public void deleteRoles(RoleDeleteParam roleSetParam) {
        roleMapper.deleteBatchIds(roleSetParam.getIdList());

        LambdaQueryWrapper<MenuRole> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(MenuRole::getRoleId,roleSetParam.getIdList());
        menuRoleService.remove(queryWrapper);
    }

}
