package com.singularity.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.dao.AttachmentMapper;
import com.singularity.dao.VehicleInformationMapper;
import com.singularity.pojo.dto.vehicle.CompareSineFitCharacteristicDTO;
import com.singularity.pojo.dto.vehicle.CompareVehicleInfoDTO;
import com.singularity.pojo.dto.vehicle.VehicleInformationSearchDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.SineFitCharacteristic;
import com.singularity.pojo.entity.VehicleInformation;
import com.singularity.service.SineFitCharacteristicService;
import com.singularity.service.VehicleInformationService;
import com.singularity.pojo.vo.vehicle.CompareSineFitCharacteristicVO;
import com.singularity.pojo.vo.vehicle.CompareVehicleInfoVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 车辆信息 服务实现类
 * </p>
 *
 * @author windsky
 * @since 2024-03-29
 */
@Service
public class VehicleInformationServiceImpl extends ServiceImpl<VehicleInformationMapper, VehicleInformation> implements VehicleInformationService {

    @Resource
    private VehicleInformationMapper vehicleInformationMapper;

    @Resource
    private AttachmentMapper attachmentMapper;

    @Resource
    private SineFitCharacteristicService sineFitCharacteristicService;


    @Override
    public List<VehicleInformation> getAllData(VehicleInformationSearchDTO vehicleInformationSearchDTO) {
        LambdaQueryWrapper<Attachment> qwa = new LambdaQueryWrapper<>();
        qwa.eq(Attachment::getOwnerType,"car_info");
        qwa.eq(Attachment::getOwnerId,vehicleInformationSearchDTO.getCarId());
        qwa.eq(Attachment::getBusinessType,vehicleInformationSearchDTO.getBusinessType());
        Attachment attachment = attachmentMapper.selectOne(qwa);
        if (attachment == null){
            return null;
        }
        LambdaQueryWrapper<VehicleInformation> qwv = new LambdaQueryWrapper();
        qwv.eq(VehicleInformation::getAttachId,attachment.getId());
        qwv.eq(VehicleInformation::getCarId,vehicleInformationSearchDTO.getCarId());
        return vehicleInformationMapper.selectList(qwv);
    }

    @Override
    public void setVehicleInformationData(VehicleInformation vehicleInformation) {
        vehicleInformationMapper.updateById(vehicleInformation);
    }

    /**
     * 对比质心及转动惯量-正弦拟合特性
     */
    @Override
    public CompareSineFitCharacteristicVO compareVehicleEvaluationInfo(CompareSineFitCharacteristicDTO dto){
        CompareSineFitCharacteristicVO compareSineFitCharacteristicVO = new CompareSineFitCharacteristicVO();
        List<CompareSineFitCharacteristicVO.CompareSineFitCharacteristicSon> tableDeadList  = new ArrayList<>();


        // 1、根据车辆idList查询对比信息
        LambdaQueryWrapper<SineFitCharacteristic> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(SineFitCharacteristic::getCarId,dto.getCarIdList());
        List<SineFitCharacteristic> compareKCEvaluationInfoBOList = sineFitCharacteristicService.list(queryWrapper);


        // 2、封装表头
        // 根据车辆id分组
        Map<Integer, List<SineFitCharacteristic>> map = compareKCEvaluationInfoBOList
                .stream().collect(Collectors.groupingBy(SineFitCharacteristic::getCarId));

        // 获取集合中的第一个元素
        List<SineFitCharacteristic> sineFitCharacteristicList = map.get(dto.getCarIdList().get(0));
        sineFitCharacteristicList.forEach(sineFitCharacteristic -> {
            CompareSineFitCharacteristicVO.CompareSineFitCharacteristicSon compareSineFitCharacteristicSon =
                    new CompareSineFitCharacteristicVO.CompareSineFitCharacteristicSon();
            compareSineFitCharacteristicSon.setTestName(sineFitCharacteristic.getName());

            CompareSineFitCharacteristicVO.CompareSineFitCharacteristicSon2 sineFitCharacteristicSon2 =
                    new CompareSineFitCharacteristicVO.CompareSineFitCharacteristicSon2();
            List<CompareSineFitCharacteristicVO.CompareSineFitCharacteristicSon2> subjectiveDataSon2List = new ArrayList<>();
            subjectiveDataSon2List.add(sineFitCharacteristicSon2);
            compareSineFitCharacteristicSon.setCompareSineFitCharacteristicSon2List(subjectiveDataSon2List);
            tableDeadList.add(compareSineFitCharacteristicSon);
        });
        compareSineFitCharacteristicVO.setTableDeadList(tableDeadList);

        // 3、封装对比数据集合
        Map<Integer, Map<String, List<String>>> data  = new HashMap<>();
        for (Integer carId : dto.getCarIdList()) {
            List<SineFitCharacteristic> paramList = map.get(carId);
            if (Objects.isNull(paramList) || paramList.size() < 1) {
                continue;
            }
            Map<String, List<String>> paramMap = new HashMap<>();
            for (SineFitCharacteristic sineFitCharacteristic : paramList) {
                List<String> list = new ArrayList<>();
                list.add(sineFitCharacteristic.getFreq());
                list.add(sineFitCharacteristic.getAmp());
                list.add(sineFitCharacteristic.getAccel());
                list.add(sineFitCharacteristic.getOffset());
                list.add(sineFitCharacteristic.getAmplitude());
                list.add(sineFitCharacteristic.getPhase());
                list.add(sineFitCharacteristic.getTotal());
                list.add(sineFitCharacteristic.getCgValue());
                paramMap.put(sineFitCharacteristic.getName(), list);
            }
            data.put(carId, paramMap);
        }
        compareSineFitCharacteristicVO.setData(data);
        return compareSineFitCharacteristicVO;
    }

    /**
     * 对比质心及转动惯量-车辆参数
     */
    @Override
    public CompareVehicleInfoVO compareVehicleInfo(CompareVehicleInfoDTO dto){
        CompareVehicleInfoVO compareVehicleInfoVO = new CompareVehicleInfoVO();
        // 1、根据车辆idList查询对比信息
        LambdaQueryWrapper<VehicleInformation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(VehicleInformation::getCarId,dto.getCarIdList());
        List<VehicleInformation> vehicleInformationList = this.list(queryWrapper);

        // 2、封装表头
        List<CompareVehicleInfoVO.CompareVehicleInfoSon> tableDeadList  = new ArrayList<>();
        // 根据车辆id分组
        Map<Integer, List<VehicleInformation>> map = vehicleInformationList
                .stream().collect(Collectors.groupingBy(VehicleInformation::getCarId));
        // 获取集合中的第一个元素
        List<VehicleInformation> firstList = map.get(dto.getCarIdList().get(0));
        firstList.forEach(vehicleInformation -> {
            CompareVehicleInfoVO.CompareVehicleInfoSon compareVehicleInfoSon = new
                    CompareVehicleInfoVO.CompareVehicleInfoSon();
            compareVehicleInfoSon.setTestName(vehicleInformation.getName());
            tableDeadList.add(compareVehicleInfoSon);
        });
        compareVehicleInfoVO.setTableDeadList(tableDeadList);

        // 3、封装对比数据集合
        Map<Integer, Map<String, String>> data  = new HashMap<>();
        for (Integer carId : dto.getCarIdList()) {
            List<VehicleInformation> paramList = map.get(carId);
            if (Objects.isNull(paramList) || paramList.size() < 1) {
                continue;
            }
            Map<String, String> paramMap = new HashMap<>();
            for (VehicleInformation vehicleInformation : paramList) {
                paramMap.put(vehicleInformation.getName(), vehicleInformation.getValue());
            }
            data.put(carId, paramMap);
        }
        compareVehicleInfoVO.setData(data);
        return compareVehicleInfoVO;
    }
}
