package com.singularity.util;

import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.springframework.stereotype.Component;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.context.Context;

import javax.annotation.Resource;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;

import org.jsoup.nodes.Document;
import org.xhtmlrenderer.pdf.ITextRenderer;

@Slf4j
@Component
public class HtmlUtil {

    @Resource
    private TemplateEngine templateEngine;

    /**
     * thymeleaf静态化生成
     *
     * @param templateName 模板名称
     * @param outFilePath 文件输出路径
     * @param map 模板映射参数
     */
    public void createHtml(String templateName, String outFilePath,Map<String, Object> map) {

        PrintWriter writer = null;
        try {

            // 模板上下文 主要存储Model参数
            Context context = new Context();
            if (map.size() > 0) {
                context.setVariables(map);
            }

            // 创建输出文件
            File folder = new File(outFilePath);
            //如果文件不存在,直接创建
            if (!folder.exists()) {
                folder.createNewFile();
            }

            // 获取输出目标文件输出流
            writer = new PrintWriter(folder, "UTF-8");

            // 生成静态模板参数1:template模板名称  参数2:上下文对象  参数3:目标文件输出流
            templateEngine.process(templateName, context, writer);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // flush输出流并关闭
            if (writer != null) {
                writer.flush();
                writer.close();
            }
        }
    }

    /**
     * html文件转换为html字符串
     *
     * @param filePathAndName 文件路径+文件名
     */
    public static String readFile(String filePathAndName) {
        String result = "";
        BufferedReader in = null;
        try {
            URL realUrl = new URL(filePathAndName);
            // 打开和URL之间的连接
            URLConnection connection = realUrl.openConnection();
            // 设置通用的请求属性
            connection.setRequestProperty("accept", "*/*");
            connection.setRequestProperty("connection", "Keep-Alive");
            connection.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 建立实际的连接
            connection.connect();
            // 获取所有响应头字段
            Map<String, List<String>> map = connection.getHeaderFields();
            // 遍历所有的响应头字段
            for (String key : map.keySet()) {

            }
            // 定义 BufferedReader输入流来读取URL的响应
            in = new BufferedReader(new InputStreamReader(
                    connection.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            System.out.println("发送GET请求出现异常！" + e);
            e.printStackTrace();
        }
        // 使用finally块来关闭输入流
        finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (Exception e2) {
                e2.printStackTrace();
            }
        }
        return result;
    }

    /**
     * htmlToPdf
     *
     * @param htmlStr html字符串
     * @param outPathAndName 输出文件路径+文件名
     */
    public static void htmlToPdf(String htmlStr, String outPathAndName){
        Document doc = Jsoup.parse(htmlStr);
        htmlStr = doc.html();
        ITextRenderer renderer = new ITextRenderer();
        renderer.setDocumentFromString(htmlStr);
        renderer.layout();
        try (OutputStream out = new FileOutputStream(outPathAndName)) {
            renderer.createPDF(out);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}
