package com.singularity.util.authentication;

import cn.hutool.core.io.IoUtil;
import cn.hutool.json.JSONUtil;
import lombok.Data;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.Base64;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * 认证工具
 * @author 天
 * Time: 2023/11/15 19:57
 */
@Data
public class AuthenticationTool {

    private String faceAppCode;

    private String facePath;

    private String idCardPath;

    public AuthenticationTool(String faceAppCode, String facePath,String idCardPath) {
        this.faceAppCode = faceAppCode;
        this.facePath = facePath;
        this.idCardPath=idCardPath;
    }

    public CommonResponse faceRecognition(FaceRecognitionParam faceRecognitionParam) throws IOException {
        String name = faceRecognitionParam.getName();
        String idcard = faceRecognitionParam.getIdcard();
        Map<String, String> params = new HashMap<>();
        params.put("idcard",idcard);
        params.put("name", name);
        //image 和 url 二选一 ，选择其中一个时，请注释另外一个参数,默认使用image
        InputStream inputStream = faceRecognitionParam.getImage().getInputStream();
        byte[] bytes = IoUtil.readBytes(inputStream);
        String s = Base64.getEncoder().encodeToString(bytes);
        inputStream.close();
        params.put("image", s);
        return postForm(this.faceAppCode, this.facePath, params);
    }

    private CommonResponse postForm(String appCode, String url, Map<String, String> params) throws IOException {
        OkHttpClient client = new OkHttpClient.Builder().build();
        FormBody.Builder formbuilder = new FormBody.Builder();
        Iterator<String> it = params.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            formbuilder.add(key, params.get(key));
        }
        FormBody body = formbuilder.build();
        Request request = new Request.Builder().url(url).addHeader("Authorization", "APPCODE " + appCode).post(body).build();
        Response response = client.newCall(request).execute();
        System.out.println("返回状态码" + response.code() + ",message:" + response.message());
        String result = response.body().string();
        return JSONUtil.toBean(result, CommonResponse.class);
    }

    public CommonResponse idCardRecognition(MultipartFile idCard) throws IOException {
        InputStream inputStream = idCard.getInputStream();
        byte[] bytes = IoUtil.readBytes(inputStream);
        String s = Base64.getEncoder().encodeToString(bytes);

        Map<String, String> params = new HashMap<>();
        inputStream.close();
        params.put("image", s);
        return postForm(this.faceAppCode, this.idCardPath, params);
    }


}
